/****************************************************************************

    PROGRAM:    mailscan

    FILE:       mailcsan.c -> mailscan.exe

    Author:	Hans-Georg Franke (hansgeorgfranke@yahoo.de)

    This file is part of OpenMailScan.

    OpenMailScan is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    OpenMailScan is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with MailScan (see license.txt); if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


****************************************************************************/

#if defined(LINUX)
extern "C" {
#endif

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

/* Lotus C API for Domino and Notes include files */

#include <stdnames.h>
#include <global.h>
#include <addin.h>
#include <nsfdb.h>
#include <nif.h>
#include <nsfnote.h>
#include <nsfsearc.h>
#include <idtable.h>
#include <editods.h>
#include <osenv.h>
#include <ostime.h>
#include <osfile.h>
#include <osmisc.h>
#include <osmem.h>
#include <mailserv.h>
#include <mail.h>
#if defined (LINUX)
#include <unistd.h>
#endif

#include "addin.h"
#include "log.h"
#include "config.h"
#include "processNote.h"

STATUS LNPUBLIC AddIDUnique(void far * phNoteIDTable,SEARCH_MATCH far *pSearchMatch,ITEM_TABLE far *summary_info);
STATUS LNPUBLIC Processnote (void far * phDB, DWORD NoteID);


//Global Variables

DBHANDLE	hlocdb;


#if defined POSIXIPC

#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/sem.h>
key_t bakey= _HGF_SEMAPHORE_LINUX;
int basemid;
struct sembuf *basops = (struct sembuf *) malloc(2 * sizeof( struct sembuf));

#else
HANDLE hMutex;
DWORD dwWaitResult;
#endif


/************************************************************************

    FUNCTION:   AddInMain

    PURPOSE:    Main entry point for all Lotus Domino Server add-in tasks

*************************************************************************/

STATUS LNPUBLIC  AddInMain (HMODULE hModule, int argc, char *argv[])
{

    // Local data.

    HANDLE		hOldStatusLine;  // handle to initial default status line
    HANDLE		hStatusLineDesc; // handle to new default status line
    HMODULE		hMod;            // add-in task's module handle
    int			n;
    STATUS		serror;
    char 		szBuffer[512];
    HANDLE 		hNoteIDTable;
    char		Version[]="0.99f";
	#if defined (POSIXIPC)
    union semun {
        int val;                    /* value for SETVAL */
        struct semid_ds *buf;       /* buffer for IPC_STAT, IPC_SET */
        unsigned short int *array;  /* array for GETALL, SETALL */
        struct seminfo *__buf;      /* buffer for IPC_INFO */
    } sem_union;
	#endif

	LogMessageText("*********************************", NOERROR);
    LogMessageText("Startup......", NOERROR);
	LogMessageText("*********************************", NOERROR);

    AddInQueryDefaults (&hMod, &hOldStatusLine);

    AddInDeleteStatusLine (hOldStatusLine);

    hStatusLineDesc = AddInCreateStatusLine("Mailscan Addin");

    AddInSetDefaults (hMod, hStatusLineDesc);

    AddInSetStatusText("Initializing");
//	AddInSetStatusText("Idle");

	#if defined(LINUX)
    sprintf(szBuffer, "PID>%d", getpid());
	LogMessageTextCon(szBuffer, NOERROR);
	#endif

    sprintf(szBuffer, "OpenMailScan Version %s, Copyright (C) 2004 Hans-Georg Franke", Version);
	LogMessageTextCon(szBuffer, NOERROR);
	sprintf(szBuffer, "OpenMailScan comes with ABSOLUTELY NO WARRANTY;");
	LogMessageTextCon(szBuffer, NOERROR);
	sprintf(szBuffer, "This is free software, and you are welcome to redistribute it under certain conditions;");
	LogMessageTextCon(szBuffer, NOERROR);
	sprintf(szBuffer, "For details see included file \"license.txt\"");
	LogMessageTextCon(szBuffer, NOERROR);
	sprintf(szBuffer, "MAXPATH=%i",MAXPATH);
	LogMessageTextCon(szBuffer, NOERROR);

	LogMessageTextCon("Fetching and signaling mutex", NOERROR);

	#if defined POSIXIPC

	//Creating mutex
	if ( ( basemid = semget(bakey, 1, 0600 | IPC_CREAT)) == -1  )
	{
		LogMessageTextSeverity("Could not create mutex", NOERROR,0);
		return (FALSE);
	}

	//Initializing mutex
	sem_union.val = 1;
	if (semctl(basemid, 0, SETVAL, sem_union) == -1)
	{
		LogMessageTextSeverity("Could not initialize mutex", NOERROR,0);
		return (FALSE);
	}


	LogMessageTextCon("Signaling mutex", NOERROR);
	//signaling mutex
	basops[0].sem_num = 0;
	basops[0].sem_op = -1;
	basops[0].sem_flg = SEM_UNDO;

	if ( semop(basemid, basops, 1) == -1  )
	{
		LogMessageTextSeverity("Cannot signal Mutex\n", NOERROR,0);
		return (FALSE);
	}
	else
	{
		LogMessageTextSeverity("Mutex signaled", NOERROR,0);
	}
	#else

	hMutex = CreateMutex(NULL,FALSE,"HGFMailScanMutex");
	dwWaitResult = WaitForSingleObject( hMutex, 5000L);
	switch (dwWaitResult)
		{
		case WAIT_OBJECT_0:
			//Got Mutex
			LogMessageTextSeverity("Mutex signaled\n", NOERROR,0);
			break;
		default:
			LogMessageTextSeverity("Cannot signal Mutex\n", NOERROR,0);
			return (FALSE);
		}

	#endif



	DebugLevel=OSGetEnvironmentInt("MailScan_DebugLevel");
 	sprintf(szBuffer, "DebugLevel: %i", DebugLevel);
	LogMessageTextCon(szBuffer, NOERROR);

	//Laden der Konfiguration aus der NOTES.INI (via config.c-loadConfig())
	loadConfig();

	LogMessageTextConSeverity("Searching for Mailboxes.", NOERROR,0);
    LogMessageText("Searching for Mailboxes.", NOERROR);

	searchMailboxes();

    sprintf(szBuffer, "Startup Completed.");
    LogMessageTextCon(szBuffer, NOERROR);

    while (!AddInIdle())
    {

        OSPreemptOccasionally();

        if (AddInSecondsHaveElapsed(10))
        {
        AddInSetStatusText("Doing 10-second job");

		for ( n=(nrMailboxes==0?0:1) ; n<=nrMailboxes; n++)
			{
				serror = IDCreateTable(sizeof(NOTEID), &hNoteIDTable);
				if (serror != NOERROR )
				{
//					for (n=(nrMailboxes==0?0:1); nrMailboxes; n++)
//					{
//						if (hdb[n] != NULLHANDLE)
//						{
//							NSFDbClose(hdb[n]);
//							hdb[n] = NULLHANDLE;
//						}
//					}

					sprintf(szBuffer,"IDCreateTable-Error");
					LogMessageTextConSeverity(szBuffer, NOERROR,1);
					sprintf(szBuffer,"Exiting this execution..............");
					LogMessageTextConSeverity(szBuffer, NOERROR,0);
					goto nextloop;
					//return (ERR(serror));
				}

				hlocdb = hdb[n];

				serror =NSFSearch (
						hdb[n],				// database handle
						NULLHANDLE,			// selection formula (select all notes)
						NULL,				// title of view in selection formula
						0,					// search flags
						NOTE_CLASS_DATA,	// note class to find
						NULL,				// starting date (unused)
						AddIDUnique,		// call for each note found
						&hNoteIDTable,		// argument to AddIDUnique
						NULL);				// returned ending date (unused)

				if ( serror != NOERROR)
				{
//					IDDestroyTable(hNoteIDTable);
//					for (l=(nrMailboxes==0?0:1); l<=nrMailboxes; l++)
//    					{
//				 		if (hdb[l] != NULLHANDLE)
//    						{
//	    						NSFDbClose(hdb[l]);
//	    						hdb[l] = NULLHANDLE;
//						}
//   				}
					sprintf(szBuffer,"NSFSearch-Error with Mailbox Nr. %i", n);
					LogMessageTextConSeverity(szBuffer, NOERROR,1);
					sprintf(szBuffer,"Exiting this execution..............");
					LogMessageTextConSeverity(szBuffer, NOERROR,0);
					break;
					goto nextloop;
				}

				if (serror = IDEnumerate(hNoteIDTable,
							Processnote,    // called for each ID
							&(hdb[n]) ))       // arg passed to func
				{
					char error_text[200];

					sprintf(szBuffer,"IDEnumerate-Error with Mailbox Nr. %i", n);
					LogMessageTextConSeverity(szBuffer, NOERROR,0);

					OSLoadString(NULLHANDLE, ERR(serror), error_text, sizeof(error_text));
					error_text[200]='\0';
					sprintf(szBuffer,"IDEnumerate failurereason %.200s", error_text);
					LogMessageTextConSeverity(szBuffer, NOERROR,0);

//					IDDestroyTable(hNoteIDTable);
//					for (l=(nrMailboxes==0?0:1); l<=nrMailboxes; l++)
//   					{
//				 		if (hdb[l] != NULLHANDLE)
//   						{
//    						NSFDbClose(hdb[l]);
//    						hdb[l] = NULLHANDLE;
//						}
//   					}

					sprintf(szBuffer,"Exiting this execution..............");
					LogMessageTextConSeverity(szBuffer, NOERROR,0);
					goto nextloop;
					//return (NOERROR);
				}


				serror = IDDestroyTable(hNoteIDTable);
				if ( serror != NOERROR)
				{
					char error_text[200];

					sprintf(szBuffer,"IDDestroy-Error with Mailbox Nr. %i", n);
					LogMessageTextConSeverity(szBuffer, NOERROR,0);

					OSLoadString(NULLHANDLE, ERR(serror), error_text, sizeof(error_text));
					error_text[200]='\0';
					sprintf(szBuffer,"IDDestroy failurereason %.200s", error_text);
					LogMessageTextConSeverity(szBuffer, NOERROR,0);

//					for (l=(nrMailboxes==0?0:1); l<=nrMailboxes; l++)
//   					{
//			 			if (hdb[l] != NULLHANDLE)
//   						{
//    						NSFDbClose(hdb[l]);
//    						hdb[l] = NULLHANDLE;
//						}
//   					}
					sprintf(szBuffer,"Exiting this execution..............");
					LogMessageTextConSeverity(szBuffer, NOERROR,0);
					goto nextloop;
					//return (NOERROR);
				}

			}
	     }

nextloop:

		AddInSetStatusText("Idle");

    }   // End of main task loop.

   //	We get here when the server notifies us that it is time to terminate.
   //   This is usually when the user has entered "quit" to the server console.
   //   Clean up anything we have been doing.
   //
    AddInSetStatusText("Terminating");


	LogMessageTextCon("Freeing mutex", NOERROR);

	#if defined POSIXIPC
	//freeing mutex
	basops[0].sem_num = 0;
	basops[0].sem_op = 1;
	basops[0].sem_flg = SEM_UNDO;

	if ( semop(basemid, basops, 1) == -1  )
	{
		LogMessageTextSeverity("Mutex not freed\n", NOERROR,0);
	}
	else
	{
		LogMessageTextSeverity("Mutex freed\n", NOERROR,0);
	}

	//Removing semaphore
	if (semctl(basemid, 0, IPC_RMID, sem_union) == -1)
        	LogMessageTextSeverity("Can't remove mutex \n", NOERROR,0);
	else
        	LogMessageTextSeverity("Mutex removed \n", NOERROR,0);

	#else
	if (ReleaseMutex(hMutex))
	{
		LogMessageTextSeverity("Mutex freeed\n", NOERROR,0);
	}
	else
	{
		LogMessageTextSeverity("Mutex not freed\n", NOERROR,0);
	}



	#endif



	for (n=(nrMailboxes==0?0:1); n<=nrMailboxes; n++)
	{
		if (hdb[n] != NULLHANDLE)
		{
			NSFDbClose(hdb[n]);
			hdb[n] = NULLHANDLE;
		}
	}

	LogMessageTextCon("*********************************", NOERROR);
    LogMessageTextCon("Termination complete.", NOERROR);
	LogMessageTextCon("*********************************", NOERROR);

   // End of add-in task.  We must "return" here rather than "exit".

    return (NOERROR);
}



STATUS LNPUBLIC AddIDUnique
            (void far * phNoteIDTable,
            SEARCH_MATCH far *pSearchMatch,
            ITEM_TABLE far *summary_info)
{
    SEARCH_MATCH SearchMatch;
    HANDLE       hNoteIDTable;
    STATUS       error;
    BOOL         flagOK;

	NOTEHANDLE	 hNote;
    WORD		field_len;
    char		field_text[1024];
    BOOL 		field_found;
    char 		szBuffer[512];

    memcpy( (char*)&SearchMatch, (char*)pSearchMatch, sizeof(SEARCH_MATCH) );

    if (!(SearchMatch.SERetFlags & SE_FMATCH)){
// Cholokov
		sprintf(szBuffer,"Search doesn't match");
		LogMessageTextConSeverity(szBuffer, NOERROR,3);
// Cholokov
        return (NOERROR);
	}


    hNoteIDTable = *((HANDLE far *)phNoteIDTable);


    if (error = NSFNoteOpen (hlocdb, SearchMatch.ID.NoteID, 0, &hNote))
    {
		sprintf(szBuffer,"Can't open mail in mail.box. Already routed?");
		LogMessageTextConSeverity(szBuffer, NOERROR,3);
        return (NOERROR);
    }


	field_found = NSFItemIsPresent ( hNote, "MailScanNoteProcessed",(WORD) strlen ("MailScanNoteProcessed"));

	if (field_found)
	{
		field_len = NSFItemGetText ( hNote,"MailScanNoteProcessed",field_text, sizeof (field_text));

		if ( strcmp(field_text,"TRUE")==0 )
		{
		sprintf(szBuffer,"MailScanNoteProcessed==TRUE found");
		LogMessageTextConSeverity(szBuffer, NOERROR,3);
		NSFNoteClose(hNote);
		return (NOERROR);
		}
	}

	NSFNoteClose(hNote);


    if (error = IDInsert(hNoteIDTable, SearchMatch.ID.NoteID, &flagOK))
    {
		sprintf(szBuffer,"Error adding NotID to NoteIDTable");
		LogMessageTextConSeverity(szBuffer, NOERROR,3);
        return (ERR(error));
    }

    return (NOERROR);
}


#if defined(LINUX)
}
#endif

