/*
    PROGRAM:    mailscan

    FILE:       config.c

    Author:	Hans-Georg Franke (hansgeorgfranke@yahoo.de)

    This file is part of OpenMailScan.

    OpenMailScan is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    OpenMailScan is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with MailScan (see license.txt); if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
    */

#if defined(LINUX)
extern "C" {
#endif
#include <stdio.h>
#include <string.h>
#include <stdlib.h>


#include "config.h"
#include "log.h"

DBHANDLE	hdb[10];
WORD		nrMailboxes=10;

char	DATADIR[MAXPATH];
char	sconfigview[MAXPATH]="searchConfig";
char    sconfigdatabase[MAXPATH]="virconf.nsf";
char    environdatabase[MAXPATH]="virconf.nsf";
char	ssearchconfigure[MAXPATH]="configuration";
char	workdir[MAXPATH]="MailScan";
char	virscanner[MAXPATH];
char	virparm[MAXPATH];
BOOL scandirectory=FALSE;

LONG rc[PARAMCOUNT +1];
char virscan[PARAMCOUNT +1][MAXPATH];
BOOL isbefore[PARAMCOUNT +1];
int	 removefile[PARAMCOUNT +1];
char removetext[PARAMCOUNT +1 ][MAXPATH];
char removemail[PARAMCOUNT +1 ][MAXPATH];
char bodytext[PARAMCOUNT +1][MAXPATH];

void loadConfig(void)
{
	BOOL		retcode;
	STATUS		sStatus=NOERROR;
	HANDLE		hconfigdb=NULLHANDLE;
    NOTEID      nidView=0;
    HCOLLECTION hCollection=NULLHCOLLECTION;
    COLLECTIONPOSITION    cp;
    DWORD       dwFound;
    HANDLE		hBuffer = NULLHANDLE;
    WORD    	wSignalFlags;
    NOTEID		*pnid;
    NOTEID		nid;
    NOTEHANDLE	nHandle;
	char 		szBuffer[MAXPATH+MAXPATH];
	char		tmpBuffer[MAXPATH+MAXPATH];
	int			n;
	int			x;
	int			retlen=0;

	x = OSGetDataDirectory(DATADIR);

	retcode=OSGetEnvironmentString("MailScanConfigDB", environdatabase, sizeof(environdatabase) -1 );

	if (retcode)
	{
		strcpy(sconfigdatabase, environdatabase);
	}

	sconfigdatabase[MAXPATH]='\0';
	sprintf(szBuffer, "Using configdb: \"%.*s\"", MAXPATH, sconfigdatabase);
	LogMessageTextSeverity(szBuffer, NOERROR, 1);

    /* Read the configuration data from config-database */
    sStatus = NSFDbOpen(sconfigdatabase, &hconfigdb);

   	if (ERR(sStatus) != NOERROR)
   	{
		LogMessageTextSeverity("Error opening configdb", NOERROR, 1);
		setDefaults();
		LogMessageText("Error finding configuration information. Using defaults!", NOERROR);
		return;
	}

	sStatus = NIFFindView(hconfigdb, sconfigview, &nidView);

	if (ERR(sStatus) != NOERROR)
	{
		LogMessageTextSeverity("Error opening view", NOERROR, 1);
		setDefaults();
		LogMessageText("Error finding configuration information. Using defaults!", NOERROR);
		return;
	}

	sStatus = NIFOpenCollection(hconfigdb,
                                hconfigdb,
                                nidView,
                                0,
                                NULLHANDLE,
                                &hCollection,
                                NULL,
                                NULL,
                                NULL,
                                NULL);

    if (ERR(sStatus) != NOERROR)
    {
		LogMessageTextSeverity("Error opening Collection", NOERROR, 1);
		setDefaults();
		LogMessageText("Error finding configuration information. Using defaults!", NOERROR);
		return;
	}

	sStatus = NIFUpdateCollection(hCollection);



    if (ERR(sStatus) != NOERROR)
    {
		LogMessageTextSeverity("Error updating configdb", NOERROR, 1);
		setDefaults();
		LogMessageText("Error finding configuration information. Using defaults!", NOERROR);
		return;
	}

	sStatus = NIFFindByName(hCollection, ssearchconfigure, FIND_PARTIAL, &cp, &dwFound);


	/*Dok-handel holen */

	if (ERR(sStatus) != NOERROR)
	{
		LogMessageTextSeverity("Error finding configdok", NOERROR, 1);
		setDefaults();
		LogMessageText("Error finding configuration information. Using defaults!", NOERROR);
		return;
	}


	do
    	{
        	sStatus = NIFReadEntries(hCollection,
                                 &cp,
                                 NAVIGATE_CURRENT,
                                 0L,
                                 NAVIGATE_NEXT,
                                 0xFFFFFFFF,
                                 READ_MASK_NOTEID,
                                 &hBuffer,
                                 NULL,
                                 NULL,
                                 &dwFound,
                                 &wSignalFlags);

		if (dwFound == 0)
		{
			// No config-dok found
			LogMessageText("No config-doc found", NOERROR);
			hBuffer=NULLHANDLE;
		}
		else if (dwFound != 1)
		{
			// Error more than one config-doks
			LogMessageText("More than one config-doc found", NOERROR);
			hBuffer=NULLHANDLE;
		}
       	if ( (sStatus == NOERROR) & (hBuffer != NULLHANDLE) )
    	{
        	pnid = (NOTEID *)OSLockObject(hBuffer);
            if (pnid != (NOTEID *)NULL)
            {
                sStatus = NSFNoteOpen(hconfigdb, pnid[0], 0, &nHandle);
                if (sStatus == NOERROR)
                {

				//VirScanCmd
                NSFNoteGetInfo(nHandle, _NOTE_ID, &nid);
				
				retlen=NSFItemGetText ( nHandle,"VirScanCmd",virscanner, sizeof (virscanner));
				virscanner[retlen]=0;
				sprintf(szBuffer, "Calling scanner with \"%.*s\"", MAXPATH, virscanner);
				LogMessageTextSeverity(szBuffer, NOERROR, 1);

				//VirScanParm
				retlen=NSFItemGetText ( nHandle,"VirScanParm",virparm, sizeof (virparm));
				virparm[retlen]=0;

				//ScanDirectory
				retlen=NSFItemGetText ( nHandle,"scandir",szBuffer, sizeof (szBuffer));
				szBuffer[retlen]=0;
				if (strcmp(szBuffer, "File") != 0)
				{
					scandirectory = TRUE;
					sprintf(szBuffer, "Calling with Param \"%.*s [DIRECTROY]\\\"", MAXPATH, virparm);
				}
				else
				{
					scandirectory = FALSE;
					sprintf(szBuffer, "Calling with Param \"%.*s [FULLFILENAME]\"", MAXPATH, virparm);
				}
				LogMessageTextSeverity(szBuffer, NOERROR,1);

				//workind girectory
				retlen=NSFItemGetText ( nHandle,"workingdir",workdir, sizeof (workdir));
				workdir[retlen]=0;
				if (strcmp(workdir,"") == 0)
				{
					strcpy(workdir,"MailScan");
				}
				sprintf(szBuffer, "Working directory %.*s", MAXPATH, workdir);
				LogMessageTextSeverity(szBuffer, NOERROR,1);

				for (n=0; n<PARAMCOUNT; n++)
				{
					sprintf(szBuffer,"rc%i",n);
					rc[n] = NSFItemGetLong ( nHandle,szBuffer,rc[n]);

					sprintf(szBuffer,"before%i",n);
					retlen=NSFItemGetText ( nHandle,szBuffer,szBuffer, sizeof (szBuffer));
					if ( !strcmp(szBuffer,"") || !strcmp(szBuffer,"1"))
						isbefore[n] = TRUE;
					else
						isbefore[n] = FALSE;

					sprintf(szBuffer,"deletefile%i",n);
					retlen=NSFItemGetText ( nHandle,szBuffer,tmpBuffer, sizeof (tmpBuffer));
					tmpBuffer[retlen]=0;

					removefile[n] = 1;
					if (strncmp(tmpBuffer,"0",1)==0)
						removefile[n] = 0;
					if (strncmp(tmpBuffer,"2",1)==0)
						removefile[n] = 2;

					sprintf(szBuffer,"removetext%i",n);
					retlen=NSFItemGetText ( nHandle,szBuffer,removetext[n], sizeof (removetext[n]));
					removetext[n][retlen]=0;
					if (strcmp(removetext[n],"") == 0 )
					{
						sprintf(removetext[n],"Suspicous attachment removed: ");
					}

					sprintf(szBuffer,"removemail%i",n);

					retlen=NSFItemGetText ( nHandle,szBuffer,removemail[n], sizeof (removemail[n]));
					removemail[n][retlen]=0;
					if (strcmp(removemail[n],"") == 0 )
					{
						sprintf(removemail[n],"noone@nowhere");
					}

					sprintf(szBuffer,"VirScan%i",n);
					retlen=NSFItemGetText ( nHandle,szBuffer,virscan[n], sizeof (virscan[n]));
					virscan[n][retlen]=0;

					sprintf(szBuffer,"BodyText%i",n);
					retlen=NSFItemGetText ( nHandle,szBuffer,bodytext[n], sizeof (bodytext[n]));
					bodytext[n][retlen]=0;

					sprintf(szBuffer, "ReturnCode%i: %d - Text: \"%.*s\"", n, rc[n], MAXPATH, virscan[n]);
					LogMessageTextSeverity(szBuffer, NOERROR,1);
					
					sprintf(szBuffer, "Handling before%i: %i", n, isbefore[n]);
					LogMessageTextSeverity(szBuffer, NOERROR,1);
					
					sprintf(szBuffer, "Deleting file with rc%i: %i", n, removefile[n]);
					LogMessageTextSeverity(szBuffer, NOERROR,1);
					
					sprintf(szBuffer, "Replacing text for rc%i: %.*s", n, MAXPATH, removetext[n]);
					LogMessageTextSeverity(szBuffer, NOERROR,1);
					
					sprintf(szBuffer, "Send mail for rc%i: %.*s", n, MAXPATH, removemail[n]);
					LogMessageTextSeverity(szBuffer, NOERROR,1);
				}

				//VirScanError
				retlen=NSFItemGetText ( nHandle,"VirScanError",virscan[PARAMCOUNT], sizeof (virscan[PARAMCOUNT]));
				virscan[PARAMCOUNT][retlen]=0;
				retlen=NSFItemGetText ( nHandle,"beforeerr",szBuffer, sizeof (szBuffer));
				szBuffer[retlen]=0;
				if ( !strcmp(szBuffer,"") || !strcmp(szBuffer,"1"))
					isbefore[PARAMCOUNT] = TRUE;
				else
					isbefore[PARAMCOUNT] = FALSE;

				removefile[PARAMCOUNT] = NSFItemGetLong ( nHandle,"deletefileerr",1);

				retlen=NSFItemGetText ( nHandle,"BodyTextErr",bodytext[PARAMCOUNT], sizeof (bodytext[PARAMCOUNT]));
				bodytext[PARAMCOUNT][retlen]=0;
				
				sprintf(szBuffer, "Other Configuration Error: \"%.*s\"", MAXPATH, virscan[PARAMCOUNT]);
				LogMessageTextSeverity(szBuffer, NOERROR,1);

				sprintf(removetext[PARAMCOUNT],"Suspicous attachment removed: ");

				retlen=NSFItemGetText ( nHandle,"removeextErr",removetext[PARAMCOUNT], sizeof (removetext[PARAMCOUNT]));
				removetext[PARAMCOUNT][retlen]=0;

				retlen=NSFItemGetText ( nHandle,"removemailErr",removemail[PARAMCOUNT], sizeof (removemail[PARAMCOUNT]));
				removemail[PARAMCOUNT][retlen]=0;

			}
                	if (nHandle != NULLHANDLE)
                	{
                    		NSFNoteClose(nHandle);
                    		nHandle = NULLHANDLE;
                	}

                	OSUnlockObject(hBuffer);
            		}

            		OSMemFree(hBuffer);
            		hBuffer = NULLHANDLE;
        	}
    	} while (wSignalFlags & SIGNAL_MORE_TO_DO);




    LogMessageText("Initialization complete.", NOERROR);
    AddInSetStatusText("Idle");

}

void setDefaults(void)
{
	int n=0;

	#if defined(NT)
	sprintf(virscanner,"d:\\programme\\OfficeScan NT\\pcscan.exe");
	sprintf(virparm,"");
	#else
	sprintf(virscanner,"/usr/local/f-prot/f-prot");
	sprintf(virparm,"-dumb -archive -silent");
	#endif

	for (n=0; n<PARAMCOUNT; n++)
	{
		rc[n] = n;
		isbefore[n] = TRUE;
		removefile[n] = FALSE;
		sprintf(virscan[n],"(Not Set)");
		sprintf(bodytext[n], "");
		sprintf(removetext[n],"Suspicous attachment removed: ");
		sprintf(removemail[n],"noone@nowhere");
	}
	rc[PARAMCOUNT] = PARAMCOUNT;
	isbefore[PARAMCOUNT] = TRUE;
	removefile[PARAMCOUNT] = FALSE;
	sprintf(virscan[PARAMCOUNT], "(Not Set)");
	sprintf(bodytext[PARAMCOUNT], "");
	sprintf(removetext[PARAMCOUNT],"Suspicous attachment %s removed");
	sprintf(removemail[PARAMCOUNT],"noone@nowhere");

	return;
}

STATUS searchMailboxes(void)
{
    WORD		x;
    char 		szMailServerName[MAXUSERNAME+1];
    char        szMailFileName[MAXUSERNAME+1];
    char        szMailBoxPath[MAXPATH+1];
	char 		szBuffer[512];
    int 		n;
    STATUS		serror;


	x = OSGetDataDirectory(DATADIR);

    /* Mehrere Mailboxen abprfen, Max 10! */

    for (n=0; n<=10 ; n++)
    {
		hdb[n]=NULLHANDLE;
	}

    for (n=10; n>0 ; n--)
    {

    	sprintf(szMailServerName,"");
    	sprintf(szMailFileName,"mail%i.box",n);

    	OSPathNetConstruct( NULL,
	                    szMailServerName,
	                    szMailFileName,
                            szMailBoxPath);

    	serror = NSFDbOpen(szMailBoxPath, &hdb[n]);


		if (serror != NOERROR)
		{
			nrMailboxes--;
		}
    }

    if (nrMailboxes == 0)
    {
    	strcpy(szMailServerName,"");
    	sprintf(szMailFileName,"mail.box");

    	OSPathNetConstruct( NULL,
	                    szMailServerName,
	                    szMailFileName,
                            szMailBoxPath);

    	serror = NSFDbOpen(szMailBoxPath, &hdb[0]);
	if (serror != NOERROR)
	{
		LogMessageTextSeverity("unable to open mail.box.", serror,1);
		return(serror);
	}
    }

    if (nrMailboxes > 0)
      sprintf(szBuffer, "Number of Mailboxes: %i.", nrMailboxes);
    else
      sprintf(szBuffer, "No Multiple Mailboxes.", nrMailboxes);

    LogMessageText(szBuffer, serror);

	return (NOERROR);
}
#if defined(LINUX)
}
#endif
