/*
    PROGRAM:    mailscan

    FILE:       log.c

    Author:	Hans-Georg Franke (hansgeorgfranke@yahoo.de)

    This file is part of OpenMailScan.

    OpenMailScan is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    OpenMailScan is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with MailScan (see license.txt); if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
    */

#if defined(LINUX)
extern "C" {
#endif

#include <stdio.h>
#include <string.h>

#include <stdnames.h>
#include <global.h>
#include <addin.h>
#include <osenv.h>
#include <ostime.h>
#include <misc.h>

#include "config.h"

#include "fileTools.h"
#include "log.h"

//global Varaiables
int DebugLevel = 0;
FILE *logFile;
int logToFile = -1;
char prefix[]="MailScan-AddIn: ";

char filename[MAXPATH];
TIMEDATE	timedate;
TFMT		DateFormat;
WORD		wNavLabelLen;
char		szNavLabel[MAXALPHATIMEDATE + 1];

void LogMessageText(char * Message, STATUS stat){
	//Meldung wird immer gedruckt
	LogMessageTextSeverity(Message, stat, 0);
}

void LogMessageTextCon(char * Message, STATUS stat){
	//Meldung wird immer gedruckt
	LogMessageTextConSeverity(Message, stat, 0);
}

void LogMessageTextSeverity(char * Message, STATUS stat, int sev){
	char sBuffer[MAXPATH+21];

	if (sev <= DebugLevel)
	{
		if (-1 == logToFile)
		{
			logToFile = OSGetEnvironmentInt("MailScan_LogToFile");
		}
		if (!logFile && (logToFile > 0)  )
		{
			OSGetEnvironmentString("MailScan_LogFileName", filename, sizeof(filename)-1);
			filename[sizeof(filename)]=0;
			if (strcmp("",filename) == 0)
			{
				OSGetDataDirectory(DATADIR);
				strcpy(filename, DATADIR);
				strcat(filename,FILESEPARATOR);
				strcat(filename,"mailscan.log");
			}
			logFile = fopen(filename, "a");
		}
		if (logFile)
		{
			OSCurrentTIMEDATE(&timedate);
			DateFormat.Date = TDFMT_FULL4;
			DateFormat.Time = TTFMT_FULL;
			DateFormat.Structure = TSFMT_DATETIME;
			ConvertTIMEDATEToText(NULL, NULL, &timedate, szNavLabel, //&DateFormat
			                     MAXALPHATIMEDATE, &wNavLabelLen);
			//Returnbuffer isn't null-terminated
			szNavLabel[wNavLabelLen] = '\0';

			fprintf(logFile, szNavLabel);
			fprintf(logFile, ": ");
			fprintf(logFile, Message);
			fprintf(logFile, "\n");
			fflush(logFile);
		}
		if (2 == logToFile)
		{
			sprintf(sBuffer,"%.20s%.*s", prefix, MAXPATH, Message);
			AddInLogMessageText(sBuffer, stat);
		}
	}
}

void LogMessageTextConSeverity(char * Message, STATUS stat, int sev){
	char sBuffer[MAXPATH+21];

	if (sev <= DebugLevel)
	{
		if (-1 == logToFile)
		{
			logToFile = OSGetEnvironmentInt("MailScan_LogToFile");
		}
		if (!logFile && (logToFile > 0)  )
		{
			OSGetEnvironmentString("MailScan_LogFileName", filename, sizeof(filename)-1);
			filename[sizeof(filename)]=0;
			if (strcmp("",filename) == 0)
			{
				OSGetDataDirectory(DATADIR);
				strcpy(filename, DATADIR);
				strcat(filename,FILESEPARATOR);
				strcat(filename,"mailscan.log");
			}
			logFile = fopen(filename, "a");
		}
		if (logFile)
		{
			OSCurrentTIMEDATE(&timedate);
			DateFormat.Date = TDFMT_FULL4;
			DateFormat.Time = TTFMT_FULL;
			DateFormat.Structure = TSFMT_DATETIME;
			ConvertTIMEDATEToText(NULL, NULL, &timedate, szNavLabel, //&DateFormat
			                     MAXALPHATIMEDATE, &wNavLabelLen);
			//Returnbuffer isn't null-terminated
			szNavLabel[wNavLabelLen] = '\0';

			fprintf(logFile, szNavLabel);
			fprintf(logFile, ": ");
			fprintf(logFile, Message);
			fprintf(logFile, "\n");
			fflush(logFile);
		}

		sprintf(sBuffer,"%.20s%.*s", prefix, MAXPATH, Message);
		AddInLogMessageText(sBuffer, stat);
	}
}

#if defined(LINUX)
}
#endif
