/*
    PROGRAM:    mailscan

    FILE:       processNote.c

    Author:	Hans-Georg Franke (hansgeorgfranke@yahoo.de)

    This file is part of OpenMailScan.

    OpenMailScan is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    OpenMailScan is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with MailScan (see license.txt); if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
    */

#if defined(LINUX)
extern "C" {
#endif


#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#if defined(LINUX)
#include <sys/wait.h>
#endif

//Lotus C API for Domino and Notes include files

#include <stdnames.h>
#include <global.h>
#include <addin.h>
#include <nsfdb.h>
#include <nif.h>
#include <nsfnote.h>
#include <nsfsearc.h>
#include <idtable.h>
#include <editods.h>
#include <mailserv.h>
#include <mail.h>

//EnumComp
#include <easycd.h>
#include <editdflt.h>
#include <colorid.h>
#include <nsfdata.h>
#include <nsferr.h>
#include <ostime.h>
#include <osmisc.h>
#include <osfile.h>
#include <osmem.h>
#include <misc.h>
#include <oserr.h>

                                                                   #include "addin.h"
#include "config.h"
#include "log.h"
#include "fileTools.h"
#include "processNote.h"


BOOL		dontcopy=FALSE;
int		nratt=0;
int		wBodyItemsToDelete=0;
int		wBodyValuesToAppend=0;
BLOCKID		pBodyItemsToDelete[MAX_BODY_ITEMS_TO_DELETE];
BODYVALUE	pBodyValuesToAppend[MAX_BODY_VALUES_TO_APPEND];


STATUS LNPUBLIC Processnote (void far * phDB, DWORD NoteID)
{
    DBHANDLE    hDB;
    STATUS      error;
    STATUS      serror;
    STATUS		sstatus;
    NOTEHANDLE  hNote;
    BLOCKID 	bid;
    BLOCKID 	bidPrevious;
    WORD 		wType;
    BLOCKID 	bidValue;
    BLOCKID 	bidValuePrevious;
    DWORD 		dwLength;
    DWORD		dwLengthPrevious;
    char 		*pszValue;
    char 		szBuffer[MAXPATH+MAXPATH+MAXPATH];
    char 		szCallBuffer[MAXPATH+MAXPATH+MAXPATH+MAXPATH];
    char 		szFileName[MAXPATH];
    char 		szFileNamePath[MAXPATH];
    char		szDirectory[MAXPATH];
    NOTEID		noteid;
    int			rcsys = 0;
	int			n;
    WORD		field_len;
    char		field_text[1024];
    BOOL 		field_found;
    char		error_text[200];
	int			virscanindependentfilenames=0;


    FILEOBJECT 	*pfo;

    hDB = *( (DBHANDLE far *)phDB );

    if (error = NSFNoteOpen (hDB,NoteID,0,&hNote))
    {
// Cholokov
		sprintf(szBuffer,"Error: can't open note");
		LogMessageTextConSeverity(szBuffer, NOERROR,3);
// Cholokov
        return (ERR(error));
    }

    field_found = NSFItemIsPresent ( hNote, "MailScanNoteProcessed",(WORD) strlen ("MailScanNoteProcessed"));

	if (field_found)
	{
		field_len = NSFItemGetText ( hNote,"MailScanNoteProcessed",field_text, sizeof (field_text));

		if ( strcmp(field_text,"TRUE")==0 )
		{
		//Doc is already processed
			sprintf(szBuffer,"Doc is already in processed");
			LogMessageTextConSeverity(szBuffer, NOERROR,3);
			NSFNoteClose(hNote);
			return (NOERROR);
		}
		if ( strcmp(field_text, "RUN")==0 )
		{
		//Doc is already in processing
// Cholokov
		sprintf(szBuffer,"Doc is already in processing");
		LogMessageTextConSeverity(szBuffer, NOERROR,3);
// Cholokov
			NSFNoteClose(hNote);
			return (NOERROR);
		}
	}

	NSFItemSetText(hNote, "MailScanNoteProcessed", "RUN", 3);

	LogMessageTextSeverity("*********************************", NOERROR,2);
	LogMessageTextSeverity("Start with new Notes", NOERROR,2);
	LogMessageTextSeverity("*********************************", NOERROR,2);


	//counting attachments
	sstatus = NSFItemInfo(hNote,
							ITEM_NAME_ATTACHMENT,
							(WORD) strlen(ITEM_NAME_ATTACHMENT),
							&bid, &wType, &bidValue, &dwLength);

	while (sstatus == NOERROR)
	{
		nratt++;
		//Getting next attachment
		bidPrevious = bid;
		sstatus = NSFItemInfoNext(hNote,
						bidPrevious,
						ITEM_NAME_ATTACHMENT,
						(WORD) strlen(ITEM_NAME_ATTACHMENT),
						&bid, &wType, &bidValue, &dwLength);
	}

	sprintf(szBuffer,"Found %i %s", nratt, nratt==1 ? "Attachment" : "Attachments");
	LogMessageTextSeverity(szBuffer, NOERROR,3);



	sstatus = NSFItemInfo(hNote,
	                        ITEM_NAME_ATTACHMENT,
				(WORD) strlen(ITEM_NAME_ATTACHMENT),
				&bid, &wType, &bidValue, &dwLength);
nratt=0;
	while (sstatus == NOERROR)
	  {
		  			int maxname=MAXPATH;

		nratt++;
		LogMessageTextSeverity("Scanning Attachment", NOERROR,2);

		pszValue = OSLockBlock(char, bidValue);

		if (pszValue == (char *) NULL)
		{
			sprintf(szBuffer,"Error locking block");
			LogMessageTextSeverity(szBuffer, NOERROR,3);
			return (NOERROR);
		}

		memset(szFileName,'\0', sizeof(szFileName));
		#if defined(NT)
			pszValue += ODSLength(_WORD);
			pfo = (FILEOBJECT *) pszValue;
			if(pfo->FileNameLength < maxname){
				 maxname=pfo->FileNameLength;
			}
			else{
				LogMessageTextSeverity("Filename too big and would be truncated\n", NOERROR,0);
			}
			pszValue += ODSLength(_FILEOBJECT);
			memcpy(szFileName, pszValue, maxname);
		#else
			pfo = (FILEOBJECT*) pszValue;
			if(pfo->FileNameLength < maxname){
				 maxname=pfo->FileNameLength;
			}
			else{
				LogMessageTextSeverity("Filename too big and would be truncated\n", NOERROR,0);
			}
			memcpy(szFileName, pszValue + sizeof(FILEOBJECT)-ODSLength(_WORD), maxname);
		#endif

		sprintf(szBuffer,"Filename %.*s", MAXPATH, szFileName);
		LogMessageTextSeverity(szBuffer, NOERROR,3);

		OSUnlockBlock(bidValue);

		NSFNoteGetInfo(hNote, _NOTE_ID, &noteid);

		strcpy(szFileNamePath, DATADIR);
		strcat(szFileNamePath,FILESEPARATOR );
		strcat(szFileNamePath,workdir);

		MKDIR( szFileNamePath MODUS);

		sprintf(szBuffer,"%d",noteid);
		strcat(szFileNamePath,FILESEPARATOR );
		strcat(szFileNamePath, szBuffer);
		memset(szDirectory,'\0',sizeof(szDirectory));
		strcpy(szDirectory, szFileNamePath);

		sprintf(szBuffer,"Creating Directory %.*s", MAXPATH, szDirectory);
		LogMessageTextSeverity(szBuffer, NOERROR,3);

		MKDIR( szDirectory MODUS);

		strcat(szFileNamePath,FILESEPARATOR);
		//changes for processing attachment independing on language name
		virscanindependentfilenames = OSGetEnvironmentInt("MailScan_UseIndependentFileNames");
		if (virscanindependentfilenames == 1)
		{
			sprintf(szBuffer,"%i",nratt);
			strcat(szFileNamePath,szBuffer);
		}
		else
		{
			strcat(szFileNamePath,szFileName);
		}


		//Verzeichnis aufraeumen
		sprintf(szBuffer,"Cleaning directory %.*s", MAXPATH, szDirectory);
		LogMessageTextSeverity(szBuffer, NOERROR,2);
		cleanDirectory(szDirectory);

		sprintf(szBuffer,"Extracting to *%.*s* from %.*s", MAXPATH, szFileNamePath, MAXPATH, szFileName);
		LogMessageTextSeverity(szBuffer, NOERROR,3);

		sstatus = NSFNoteExtractFile(hNote, bid, szFileNamePath, NULL);

		if (sstatus != NOERROR)
		{
			sprintf(szBuffer,"Error extracting file to %.*s", MAXPATH, szFileNamePath);
			LogMessageTextSeverity(szBuffer, sstatus,3);
			sprintf(szBuffer,"Assuming returncode: %.2i", rcsys);
			LogMessageTextSeverity(szBuffer, NOERROR,2);
			rcsys = 0;
		}
		else
		{
			memset(szCallBuffer,'\0', sizeof(szCallBuffer));
			#if defined (LINUX)
			if (scandirectory)
				sprintf(szCallBuffer,"\"%.*s\" %.*s \"%.*s/\"", MAXPATH, virscanner, MAXPATH, virparm, MAXPATH, szDirectory);
			else
				sprintf(szCallBuffer,"\"%.*s\" %.*s \"%.*s\"", MAXPATH, virscanner, MAXPATH, virparm, MAXPATH, szFileNamePath);
			#else
			if (scandirectory)
				sprintf(szCallBuffer,"\"\"%.*s\" %.*s \"%.*s\\\"\"", MAXPATH, virscanner, MAXPATH, virparm, MAXPATH, szDirectory);
			else
				sprintf(szCallBuffer,"\"\"%.*s\" %.*s \"%.*s\"\"", MAXPATH, virscanner, MAXPATH, virparm, MAXPATH, szFileNamePath);
			#endif

			//Printing out the command
			LogMessageTextSeverity("Starting scanner with command:", NOERROR,3);
			LogMessageTextSeverity(szCallBuffer, NOERROR,2);

			//running system call
			rcsys = system(szCallBuffer);

			#if defined (LINUX)
			rcsys = WEXITSTATUS(rcsys);
			//printing out the returncode for linux
			sprintf(szBuffer,"Returncode Linux Virscan: %.2i", rcsys);
			LogMessageTextSeverity(szBuffer, NOERROR,2);
			#else
			sprintf(szBuffer,"Returncode of virusscanner: %.2i", rcsys);
			LogMessageTextSeverity(szBuffer, NOERROR,2);
			#endif
		}

		sprintf(szBuffer,"Processing returncode");
		LogMessageTextSeverity(szBuffer, NOERROR, 2);

		sstatus = processRC(rcsys, hNote);

		sprintf(szBuffer,"Getting next attachment");
		LogMessageTextSeverity(szBuffer, NOERROR, 2);

		//Getting next attachment
		bidPrevious = bid;
		bidValuePrevious = bidValue;
		dwLengthPrevious = dwLength;
		sstatus = NSFItemInfoNext(hNote,
					bidPrevious,
					ITEM_NAME_ATTACHMENT,
					(WORD) strlen(ITEM_NAME_ATTACHMENT),
					&bid, &wType, &bidValue, &dwLength);

		sprintf(szBuffer,"Processing delete for returncode %.2i", rcsys);
		LogMessageTextSeverity(szBuffer, NOERROR, 2);

		//Error not rcsys, use rcode
		for (n=0; n < PARAMCOUNT; n++)
		{
			if (rcsys == rc[n])
			{
				sprintf(szBuffer,"Processing delete for returncode-index %.2i", n);
				LogMessageTextSeverity(szBuffer, NOERROR, 2);
				processDelete(n, hNote, szFileName, bidPrevious, hDB);
			}
		}


	} //End of while

	sprintf(szBuffer,"Attachments ready");
	LogMessageTextSeverity(szBuffer, NOERROR,3);

	serror=NSFItemSetText(hNote, "MailScanNoteProcessed", "TRUE", 4);
	if (serror != NOERROR)
	{
		OSLoadString(NULLHANDLE, ERR(serror), error_text, sizeof(error_text));

		sprintf(szCallBuffer,"Item \"MailScanNoteProcessed\" %.200", error_text);
		LogMessageTextSeverity(szCallBuffer, NOERROR,3);
		NSFNoteClose(hNote);
		return(serror);
	}

	serror=NSFItemDelete(hNote,"RoutingState", 12);
	if (serror != NOERROR)
	{
		if (serror != ERR_ITEM_NOT_FOUND)
		{
			OSLoadString(NULLHANDLE, ERR(serror), error_text, sizeof(error_text));

			sprintf(szBuffer,"Delete routingstate %s.200", error_text);
			LogMessageTextSeverity(szBuffer, NOERROR,3);
			NSFNoteClose(hNote);
			return(serror);
		}
		else
		{
			sprintf(szBuffer,"Delete routingstate - item not found");
			LogMessageTextSeverity(szBuffer, NOERROR,3);
		}
	}

	serror=NSFItemDelete(hNote,"FailureReason", 13);
	if (serror != NOERROR)
	{
		if (serror != ERR_ITEM_NOT_FOUND)
		{
			OSLoadString(NULLHANDLE, ERR(serror), error_text, sizeof(error_text));

			sprintf(szBuffer,"Delete failurereason %.200s", error_text);
			LogMessageTextSeverity(szBuffer, NOERROR,3);
			NSFNoteClose(hNote);
			return(serror);
		}
		else
		{
			sprintf(szBuffer,"Delete failurereason - item not found");
			LogMessageTextSeverity(szBuffer, NOERROR,3);
		}
	}

	//Verzeichnis loeschen

	sprintf(szBuffer,"Removing directory %.*s", MAXPATH, szDirectory);
	LogMessageTextSeverity(szBuffer, NOERROR,3);

	rmDirectory(szDirectory);


	serror=NSFNoteUpdate(hNote, 0);
	if (serror != NOERROR)
	{
		OSLoadString(NULLHANDLE, ERR(serror), error_text, sizeof(error_text));

		sprintf(szBuffer,"NoteUpdate %.200s", error_text);
		LogMessageText(szBuffer, NOERROR);
		NSFNoteClose(hNote);
		return(serror);
	}

	NSFNoteClose(hNote);

	LogMessageTextSeverity("*********************************", NOERROR,2);
	LogMessageTextSeverity("processNote ready", NOERROR,2);
	LogMessageTextSeverity("*********************************", NOERROR,2);

	return(NOERROR);
}



STATUS processRC(int rcode, NOTEHANDLE hNote)
{
	STATUS serror;
	BOOL processed = FALSE;
	int n;
	char	szBuffer[256];

	sprintf(szBuffer,"Start of processRC");
	LogMessageTextSeverity(szBuffer, NOERROR, 2);

	for (n=0; n < PARAMCOUNT; n++)
	{
		if (rcode == rc[n])
		{
			sprintf(szBuffer,"Processing Subject");
			LogMessageTextSeverity(szBuffer, NOERROR, 2);
			//Changed rcode to n
			processSubject(hNote, isbefore[n], virscan[n], n);

			sprintf(szBuffer,"Processing Body");
			LogMessageTextSeverity(szBuffer, NOERROR, 2);
			serror=processBody(hNote, bodytext[n], n);
			processed = TRUE;
		}
	}

	if (!processed)
	{
		processSubject(hNote, isbefore[PARAMCOUNT], virscan[PARAMCOUNT], PARAMCOUNT);
		serror=processBody(hNote, bodytext[PARAMCOUNT], PARAMCOUNT);
	}
	return(serror);
}


STATUS processDelete(int rcode, NOTEHANDLE hNote, char szFileName[MAXPATH], BLOCKID bidAttachment, DBHANDLE hDB)
{
	STATUS	serror=NOERROR;
	char	szBuffer[MAXPATH*2];
	STATUS				status;
    CONVERTCONTEXT		Context;
    #if defined (NT)
    FARPROC				lpProc;
    #endif	//ProcessOneCDRecord function address
    HANDLE				hNewBody=NULLHANDLE;
	HANDLE				hNewBody2=NULLHANDLE;
    ActionRoutinePtr    actionroutine_ptr;
    DWORD				dwNewBodyLen;
	BLOCKID 	bidBody;
    BLOCKID 	bidBodyPrevious;
    WORD 		wBodyType;
    BLOCKID 	bidBodyValue;
    BLOCKID 	bidBodyValuePrevious;
    DWORD 		dwBodyLength;
	DWORD		dwBodyLengthPrevious;
    BODYVALUE   bvalNewBody, bvalNewBody2;
	int			i;
    WORD        wBuffLen;   //required buffer length
    WORD        wStringLen;
    WORD        wStringBufLen;


	sprintf(szBuffer,"Start of processDelete");
	LogMessageTextSeverity(szBuffer, NOERROR,3);

	if ( removefile[rcode] > 0)
	{

		wStringLen = (WORD) strlen(removetext[rcode]) + MAXPATH;
		wStringBufLen = wStringLen + (wStringLen%2);

		wBodyItemsToDelete=0;
		wBodyValuesToAppend=0;

		sprintf(szBuffer,"Processing Bodies");
		LogMessageTextSeverity(szBuffer, NOERROR,3);

		//Search all body items for bitmap, calculate new Body-Item
		status = NSFItemInfo(hNote,
		                        "Body",
								(WORD) strlen("Body"),
								&bidBody, &wBodyType, &bidBodyValue, &dwBodyLength);

		if (status == ERR_ITEM_NOT_FOUND)
		{
			sprintf(szBuffer,"No Body item found");
			LogMessageTextSeverity(szBuffer, NOERROR,2);
			//No Body item to process
			return(NOERROR);
		}
		while (status == NOERROR)
		{
			//Body item to process found

			Context.fUseBody2 = FALSE;
			Context.rcode = rcode;

			sprintf(Context.szFileName, "%.*s", MAXPATH, szFileName);

			//Allocate a block of memory to hold the new rich text item.
			//First calculate new size

			wBuffLen =  ODSLength( _CDTEXT ) + wStringBufLen;

			dwNewBodyLen = dwBodyLength + (nratt * wBuffLen);

			if (dwNewBodyLen <= OVERFLOWSIZE)
			{
				Context.dwNewBodyLen = dwNewBodyLen;
				Context.dwNewBody2Len = 0;
			}
			else
			{
				Context.dwNewBodyLen = MAXONESEGSIZE;   //appx. 64K
				Context.dwNewBody2Len = OVERFLOWSIZE;   //appx. 40K
			}

			if (status = OSMemAlloc( 0, Context.dwNewBodyLen, &hNewBody ))
			{
				sprintf(szBuffer,"malloc-Error hNewBody");
				LogMessageTextSeverity(szBuffer, NOERROR,3);
				return( ERR_MEMORY );
			}

			if (Context.dwNewBody2Len != 0)
			{
				if (status = OSMemAlloc( 0, Context.dwNewBody2Len, &hNewBody2 ))
				{
					sprintf(szBuffer,"malloc-Error hNewBody2");
					LogMessageTextSeverity(szBuffer, NOERROR,3);
					OSMemFree( hNewBody );
					return( ERR_MEMORY );
				}
			}

			Context.pNewBody = Context.pNewBodyCurrPos = OSLock( BYTE, hNewBody );

			*((WORD*)(Context.pNewBodyCurrPos)) = TYPE_COMPOSITE;
			((WORD*)(Context.pNewBodyCurrPos))++;

			if (Context.dwNewBody2Len != 0)
			{
				Context.pNewBody2 = Context.pNewBody2CurrPos = OSLock( BYTE, hNewBody2 );
				*((WORD*)(Context.pNewBody2CurrPos)) = TYPE_COMPOSITE;
				((WORD*)(Context.pNewBody2CurrPos))++;
			}

			#if defined (NT)
			lpProc = MakeProcInstance( (FARPROC) ProcessOneCDRecord, hInst );
			actionroutine_ptr = (ActionRoutinePtr)lpProc;
			#else
			actionroutine_ptr = (ActionRoutinePtr) ProcessOneCDRecord;
			#endif

			sprintf(szBuffer,"Calling EnumCompositeBuffer");
			LogMessageTextSeverity(szBuffer, NOERROR,3);

			status = EnumCompositeBuffer( bidBodyValue, dwBodyLength, actionroutine_ptr, &Context );

			#if defined(NT)
			FreeProcInstance(lpProc);
			#endif

			if (status)
			{
				sprintf(szBuffer,"Error enumerating CompositeBuffer");
				LogMessageTextSeverity(szBuffer, NOERROR,3);
				OSUnlock( hNewBody );
				OSMemFree( hNewBody );
				if (Context.dwNewBody2Len != 0)
				{
					OSUnlock( hNewBody2 );
					OSMemFree( hNewBody2 );
				}
				return( NOERROR );
			}


			if (wBodyItemsToDelete < MAX_BODY_ITEMS_TO_DELETE)
			{
				pBodyItemsToDelete[wBodyItemsToDelete++] = bidBody;
			}


			if (wBodyValuesToAppend < MAX_BODY_VALUES_TO_APPEND)
			{
				bvalNewBody.bid.pool = hNewBody;
				bvalNewBody.bid.block = NULLBLOCK;
				bvalNewBody.dwLen = (DWORD) (Context.pNewBodyCurrPos - Context.pNewBody) ;
				pBodyValuesToAppend[wBodyValuesToAppend++] = bvalNewBody;
			}
			else
			{
				sprintf(szBuffer,"New Body is too large");
				LogMessageTextSeverity(szBuffer, NOERROR,3);
				OSUnlock( hNewBody );
				OSMemFree( hNewBody );
				if (Context.dwNewBody2Len != 0)
				{
					OSUnlock( hNewBody2 );
					OSMemFree( hNewBody2 );
				}
				return( NOERROR );
			}
			if (Context.fUseBody2)
			{
				if (wBodyValuesToAppend < MAX_BODY_VALUES_TO_APPEND)
				{
					bvalNewBody2.bid.pool = hNewBody2;
					bvalNewBody2.bid.block = NULLBLOCK;
					bvalNewBody2.dwLen = (DWORD) (Context.pNewBody2CurrPos -
											Context.pNewBody2) ;
					pBodyValuesToAppend[wBodyValuesToAppend++] = bvalNewBody2;
				}
				else
				{
					sprintf(szBuffer,"New Body is too large - 2");
					LogMessageTextSeverity(szBuffer, NOERROR,3);
					OSUnlock( hNewBody );
					OSMemFree( hNewBody );
					if (Context.dwNewBody2Len != 0)
					{
						OSUnlock( hNewBody2 );
						OSMemFree( hNewBody2 );
					}
					return( NOERROR );
				}
			}

			OSUnlock( hNewBody );
			if (Context.dwNewBody2Len != 0)
			{
				OSUnlock( hNewBody2 );
			}

			//Getting next body item
			bidBodyPrevious = bidBody;
			bidBodyValuePrevious = bidBodyValue;
			dwBodyLengthPrevious = dwBodyLength;
			status = NSFItemInfoNext(hNote,
						bidBodyPrevious,
						"Body",
						(WORD) strlen("Body"),
						&bidBody, &wBodyType, &bidBodyValue, &dwBodyLength);
			if (status == ERR_ITEM_NOT_FOUND)
			{
				sprintf(szBuffer,"Ready with body-items");
				LogMessageTextSeverity(szBuffer, NOERROR,3);
				status = NOERROR;
				break;
			}

		} // end of while (status == NOERROR)

		sprintf(szBuffer,"Deleting old Body-Items");
		LogMessageTextSeverity(szBuffer, NOERROR,3);

		//Loop through all Bodyitems, delete all old and replace with new ones
	    for (i = 0; i < wBodyItemsToDelete; i++)
		{
			if (status = NSFItemDeleteByBLOCKID( hNote,
								pBodyItemsToDelete[i] ))
			{
				//Unable to delete old body from response document
				sprintf(szBuffer,"Unable to delete Body item");
				LogMessageTextSeverity(szBuffer, NOERROR,2);
				break;
			}
		}
		if (status)
		{
			for (i = 0; i < wBodyValuesToAppend; i++)
			{
				OSMemFree( pBodyValuesToAppend[i].bid.pool );
			}
			return (NOERROR);
		}


		sprintf(szBuffer,"Appending new Body-Items");
		LogMessageTextSeverity(szBuffer, NOERROR,3);
		//Append all new body items identified in pBodyValuesToAppend[]

		for (i = 0; i < wBodyValuesToAppend; i++)
		{
			if (status = NSFItemAppendByBLOCKID( hNote, 0,
								"Body",
								(WORD) strlen("Body"),
								pBodyValuesToAppend[i].bid,
								pBodyValuesToAppend[i].dwLen,
								NULL ))
			{
				//Unable to append new body item to response note
				sprintf(szBuffer,"Unable to append new Body item");
				LogMessageTextSeverity(szBuffer, NOERROR,3);
				break;
			}
		}

		if (status)
		{
			for ( ; i < wBodyValuesToAppend; i++ )
			{
				OSMemFree( pBodyValuesToAppend[i].bid.pool );
			}
		}

		sprintf(szBuffer,"Ready Appending new Body-Items");
		LogMessageTextSeverity(szBuffer, NOERROR,3);


		sprintf(szBuffer,"Remove-filecode %.2i", removefile[rcode]);
		LogMessageTextSeverity(szBuffer, NOERROR,3);

		if ( removefile[rcode] == 1)
		{
			char	*szFileName=NULL;
			NOTEID	noteid;
			char	*szFileBuffer=NULL;
			char	szNoteidBuffer[256];
			char	inputmailfrom[MAXPATH];
			char	mailfrom[MAXPATH];
			int	n,l=0;
			DWORD	wLength;
			int		maxmsg=50;
			char	szBuffer[MAXPATH*3];


			sprintf(szBuffer, "Moving Attachment");
			LogMessageTextSeverity(szBuffer, NOERROR,3);

			NSFNoteGetInfo(hNote, _NOTE_ID, &noteid);
			sprintf(szNoteidBuffer,"%d",noteid);

			NSFItemGetText(hNote, "FROM", inputmailfrom, sizeof(inputmailfrom));

			for (n=0; n<= (int)strlen(inputmailfrom); n++)
			{
				if (inputmailfrom[n] == '/' || inputmailfrom[n] =='\\')
				{
					mailfrom[l++] = '_';
				}
				else
				{
					mailfrom[l++] = inputmailfrom[n];
				}

			}

			sprintf(szBuffer, "New Mail From: %.*s ", MAXPATH, mailfrom);
			LogMessageTextSeverity(szBuffer, NOERROR,3);

			wLength = (DWORD) strlen(DATADIR) + 3 * strlen(FILESEPARATOR) + strlen(workdir)
				  + strlen(mailfrom) + strlen(szNoteidBuffer) + strlen(Context.szFileName)
				  + strlen(Context.szOrigFileName) + 5 + 3 * 2 +1;

			sprintf(szBuffer,"filename length %.3i", wLength);
			LogMessageTextSeverity(szBuffer, NOERROR,3);

			szFileName = (char *) malloc( wLength * sizeof(char));
			if (szFileName == NULL)
			{
				sprintf(szBuffer,"Error getting memory");
				LogMessageTextSeverity(szBuffer, NOERROR,3);
				return(NOERROR);
			}

			memset(szFileName,'\0', sizeof(szFileName));

			strcpy(szFileName, DATADIR);
			strcat(szFileName,FILESEPARATOR );
			strcat(szFileName,workdir);
			strcat(szFileName,FILESEPARATOR );
			strcat(szFileName,"quara");
			//dangerous operation because szBuffer is fixed for length
			if (sizeof(szBuffer) <= (sizeof(szFileName)+maxmsg)){
				LogMessageTextSeverity("szBuffer overflow - can't display file name (part1)", NOERROR,0);
			}else{
				sprintf(szBuffer, "New filename (part1) : %s ", szFileName);
				LogMessageTextSeverity(szBuffer, NOERROR,3);
			}

			MKDIR(szFileName MODUS);

			strcat(szFileName,FILESEPARATOR );
			strcat(szFileName,mailfrom);
			strcat(szFileName,"--");
			strcat(szFileName, szNoteidBuffer);
			strcat(szFileName,"--");
			strcat(szFileName,Context.szFileName);
			strcat(szFileName,"--");
			if (strcmp(Context.szOrigFileName, "")==0)
			{
				strcat(szFileName,Context.szFileName);
			}
			else
			{
				strcat(szFileName,Context.szOrigFileName);
			}

			//dangerous operation because szBuffer is fixed for length
			if (sizeof(szBuffer) <= (sizeof(szFileName)+maxmsg)){
				LogMessageTextSeverity("szBuffer overflow - can't display file name (part2)", NOERROR,0);
			}else{
				sprintf(szBuffer, "New filename (part2) : %s ", szFileName);
				LogMessageTextSeverity(szBuffer, NOERROR,3);
			}

			//Using szFileBuffer because i don't know the filenamelength!
			wLength += (DWORD) strlen("Extracting attachment  for returncode ")
				  + strlen(szFileName) + 3;
			szFileBuffer = (char *) malloc( wLength * sizeof(char));

			sprintf(szFileBuffer, "Extracting attachment %.*s for returncode %.2i",	MAXPATH, szFileName, rcode);
			LogMessageTextSeverity(szFileBuffer, NOERROR,2);

			serror = NSFNoteExtractFile(hNote, bidAttachment, szFileName, NULL);

			if (szFileName != NULL)
				free(szFileName);

			if (szFileBuffer != NULL)
				free(szFileBuffer);

		} //if ( removefile[rcode] == 1)

		if ( removefile[rcode] == 2)
		{
			STATUS		error;
			HANDLE		hBodyItem;
			DWORD		bodysize;
			NOTEHANDLE	hMail;
			char		mailfrom[MAXPATH];
			char 		*szSubject;
			char		*szFileName;
			DWORD		wLength;
			char		szNoteidBuffer[256];
			NOTEID		noteid;
			char		text[255];
			int			maxmsg=50;
			char		szBuffer[MAXPATH*3];

			sprintf(szBuffer, "Mailing Attachment");
			LogMessageTextSeverity(szBuffer, NOERROR,3);

			NSFNoteGetInfo(hNote, _NOTE_ID, &noteid);
			sprintf(szNoteidBuffer,"%d",noteid);

			NSFItemGetText(hNote, "FROM", mailfrom, sizeof(mailfrom));

			if (error = NSFNoteCreate (hDB, &hMail))
			{
				return (ERR(error));
			}

			wLength = (DWORD) (strlen(DATADIR) + 3 * strlen(FILESEPARATOR) + strlen(workdir) +
				  strlen(szNoteidBuffer) + strlen(Context.szOrigFileName) + 1 );

			sprintf(szBuffer,"filename length %.3i", wLength);
			LogMessageTextSeverity(szBuffer, NOERROR,3);

			szFileName = (char *) malloc( wLength * sizeof(char));

			if (szFileName == NULL)
			{
				sprintf(szBuffer,"Error getting memory");
				LogMessageTextSeverity(szBuffer, NOERROR,3);
				return(NOERROR);
			}

			sprintf(szBuffer, "resetting filename");
			LogMessageTextSeverity(szBuffer, NOERROR,3);

			memset(szFileName,'\0', sizeof(szFileName));

			strcpy(szFileName, DATADIR);
			strcat(szFileName,FILESEPARATOR );
			strcat(szFileName,workdir);
			strcat(szFileName,FILESEPARATOR );
			strcat(szFileName, szNoteidBuffer);

			if (sizeof(szBuffer) <= (sizeof(szFileName)+maxmsg)){
				LogMessageTextSeverity("szBuffer overflow - can't display file name (part1)", NOERROR,0);
			}else{
				sprintf(szBuffer, "Temporary filename (part1) : %s ", szFileName);
				LogMessageTextSeverity(szBuffer, NOERROR,3);
			}

			MKDIR(szFileName MODUS);

			strcat(szFileName,FILESEPARATOR );
			strcat(szFileName,Context.szOrigFileName);

			if (sizeof(szBuffer) <= (sizeof(szFileName)+maxmsg)){
				LogMessageTextSeverity("szBuffer overflow - can't display file name (part2)", NOERROR,0);
			}else{
				sprintf(szBuffer, "Temporary filename (part2) : %s ", szFileName);
				LogMessageTextSeverity(szBuffer, NOERROR,3);
			}

			szSubject = (char *) malloc( (strlen ("Virus: ") + strlen(Context.szOrigFileName) + strlen(" from ") + strlen(mailfrom) + 1 ) * sizeof(char));
			if (NULL == szSubject)
			{
				NSFNoteClose(hMail);
				return (ERR(error));
			}

			sprintf(szBuffer, "Virus: %.*s from %.*s", MAXPATH, Context.szOrigFileName, MAXPATH, mailfrom);
			error=NSFItemSetText(hMail, "Subject", szBuffer, (WORD) strlen(szBuffer));
			free(szSubject);
			if (error != NOERROR)
			{
				NSFNoteClose(hMail);
				return (ERR(error));
			}

			strcpy(text,"Memo");
			if (error = NSFItemAppend(hMail, ITEM_SUMMARY,
							"Form", (WORD) strlen("Form"),
							TYPE_TEXT,text,
							(DWORD) strlen(text) ) )
			{
				NSFNoteClose(hMail);
				return (ERR(error));
			}

			if (error = NSFItemAppend(hMail, ITEM_SUMMARY,
							"Recipients", (WORD) strlen("Recipients"),
							TYPE_TEXT,removemail[rcode],
							(DWORD) strlen(removemail[rcode]) ) )
			{
				NSFNoteClose(hMail);
				return (ERR(error));
			}

			if (error = NSFItemAppend(hMail, ITEM_SUMMARY,
							"SendTo", (WORD) strlen("SendTo"),
							TYPE_TEXT,removemail[rcode],
							(DWORD) strlen(removemail[rcode]) ) )
			{
				NSFNoteClose(hMail);
				return (ERR(error));
			}

			strcpy(text, "MailScan-System");
			if (error = NSFItemAppend(hMail, ITEM_SUMMARY,
							"From", (WORD) strlen("From"),
							TYPE_TEXT, text,
							(DWORD) strlen(text) ) )
			{
				NSFNoteClose(hMail);
				return (ERR(error));
			}

			error=MailCreateBodyItem(&hBodyItem, &bodysize);
			if (error != NOERROR)
			{
				NSFNoteClose(hMail);
				return(error);
			}

			strcpy(text, "Removed Attachment:\n");
			error=MailAppendBodyItemLine(hBodyItem, &bodysize,
						text,
						(WORD) strlen(text));
			if (serror != NOERROR)
			{
				NSFNoteClose(hMail);
				OSMemFree(hBodyItem);
				return(error);
			}

			error=MailAddBodyItemExt(hMail, hBodyItem, bodysize, NULL);
			if (error != NOERROR)
			{
				NSFNoteClose(hMail);
				OSMemFree(hBodyItem);
				return(error);
			}

			//Adding attachment
			error = NSFNoteExtractFile(hNote, bidAttachment, szFileName, NULL);
			if (error != NOERROR)
			{
				NSFNoteClose(hMail);
				OSMemFree(hBodyItem);
				return(error);
			}

			error = NSFNoteAttachFile(hMail,
						ITEM_NAME_ATTACHMENT,
						(WORD) strlen(ITEM_NAME_ATTACHMENT),
						szFileName, //filename in filesystem
						Context.szOrigFileName, //original filename
						COMPRESS_HUFF);
			if (error != NOERROR)
			{
				NSFNoteClose(hMail);
				OSMemFree(hBodyItem);
				return(error);
			}

			strcpy(text, "TRUE");
			if (error = NSFItemAppend(hMail, ITEM_SUMMARY,
							"MailScanNoteProcessed", (WORD) strlen("MailScanNoteProcessed"),
							TYPE_TEXT,text,
							(DWORD) strlen(text) ) )
			{
				NSFNoteClose(hMail);
				OSMemFree(hBodyItem);
				return (ERR(error));
			}

			error = NSFNoteUpdate(hMail, 0);
			if (error != NOERROR)
			{
				NSFNoteClose(hMail);
				OSMemFree(hBodyItem);
				return(error);
			}

			NSFNoteClose(hMail);
			OSMemFree(hBodyItem);

		} //if ( removefile[rcode] == 2)

		sprintf(szBuffer, "Removing Attachment %.*s from mail for returncode %.2i", MAXPATH, Context.szOrigFileName, rcode);
		LogMessageTextSeverity(szBuffer, NOERROR,2);

 		serror = NSFNoteDetachFile(hNote, bidAttachment);
	}



	return(serror);

}



STATUS  LNCALLBACK  ProcessOneCDRecord(char   *RecordPtr, WORD    RecordType, DWORD   RecordLength, void   *pvoidContext )
{
	STATUS			status;
    CONVERTCONTEXT *pContext;
    BYTE           *pDest;
    BYTE           *pSource=NULL;

	DWORD           dwCount;
	char			szBuffer[MAXPATH*2];


	sprintf(szBuffer,"ProcessOneCDRecord");
	LogMessageTextSeverity(szBuffer, NOERROR,3);

	pContext = (CONVERTCONTEXT *)pvoidContext;
   	pDest = pContext->pNewBodyCurrPos;  //assume no overflow yet

	//if we are handling a CD_HOTSPOT check it
	if (dontcopy==TRUE)
	{
		if (RecordType == SIG_CD_HOTSPOTEND)
		{
			sprintf(szBuffer,"End of SIG_CD_HOTSPOT found");
			LogMessageTextSeverity(szBuffer, NOERROR,3);
			dontcopy=FALSE;
		}
		return (NOERROR);
	}


	if (RecordType == SIG_CD_HOTSPOTBEGIN)
	{
		char far		*p = RecordPtr;
		CDHOTSPOTBEGIN  cdHotSpot;
		char			*filename=NULL;
		char			*orgfilename=NULL;
		int				length;
		char			*szString;
		WORD			wStringLen;
		WORD			wStringBufLen;
		union {
					FONTIDFIELDS   fontFields;
					FONTID         fontId;
		} fontData;
		CDTEXT          cdtext;     //rich-text text header
		HANDLE			hcdtext;
		BYTE			*pcdtext;

		sprintf(szBuffer,"SIG_CD_HOTSPOTBEGIN found");
		LogMessageTextSeverity(szBuffer, NOERROR,3);

		ODSReadMemory( &p, _CDHOTSPOTBEGIN, &cdHotSpot, 1 );

		if (cdHotSpot.Type == HOTSPOTREC_TYPE_FILE )
		{
			sprintf(szBuffer,"HOTSPOTREC_TYPE_FILE found");
			LogMessageTextSeverity(szBuffer, NOERROR,3);
			length = (int) strlen(p);
			sprintf(szBuffer,"found length one: %.3i", length);
			LogMessageTextSeverity(szBuffer, NOERROR,3);


			//inkl. der abschliessenden Null kopieren
			filename = (char *) malloc( (length + 1 )* sizeof(char) );
			if (filename==NULL)
			{
				sprintf(szBuffer, "Error getting memory for filename");
				LogMessageTextSeverity(szBuffer, NOERROR,1);
				return(NOERROR);
			}
			memcpy(filename, p, (length + 1 )* sizeof(char));
			//dangerous operation because szBuffer is fixed for length
			sprintf(szBuffer,"attachment filename: %.*s", MAXPATH, filename);
			LogMessageTextSeverity(szBuffer, NOERROR,3);


			//increase pointer here you can find the original Name of the attachment!
			p += ( length + 1 )* sizeof(char);

			if ( strcmp(filename, pContext->szFileName) == 0)
			{
				length = (int) strlen(p);
				sprintf(szBuffer,"found file - length two: %.3i", length);
				LogMessageTextSeverity(szBuffer, NOERROR,3);

				//inkl. der abschliessenden Null kopieren
				orgfilename = (char *) malloc( (length + 1 )* sizeof(char) );
				if (orgfilename==NULL)
				{
					sprintf(szBuffer, "Error getting memory for original filename");
					LogMessageTextSeverity(szBuffer, NOERROR,1);
					if (filename != NULL)
						free(filename);
					return(NOERROR);
				}
				memcpy(orgfilename, p, (length + 1 )* sizeof(char));

				if ( strcmp(orgfilename, "") == 0)
				{
					orgfilename = (char *) realloc(orgfilename, sizeof(filename) );
					memcpy(orgfilename, filename, sizeof(filename));
				}

				//Storing original filename in Context-buffer
				sprintf(pContext->szOrigFileName, "%.*s", MAXPATH, orgfilename);

				sprintf(szBuffer,"original filename: %.*s", MAXPATH, orgfilename);
				LogMessageTextSeverity(szBuffer, NOERROR,3);

				wStringLen = (WORD) (strlen( removetext[pContext->rcode]) + strlen(orgfilename) + 4); //3 leerzeichen + 1
				wStringBufLen = wStringLen + (wStringLen%2);//Space to allocate (must be an even number of bytes)

				szString = (char *) malloc( wStringBufLen * sizeof(char));
				if (szString==NULL)
				{
					sprintf(szBuffer, "Error getting memory for new replacing text");
					LogMessageTextSeverity(szBuffer, NOERROR,1);
					if (filename != NULL)
						free(filename);
					if (orgfilename != NULL)
						free(orgfilename);
					return(NOERROR);
				}

				sprintf(szString, " %s %s ", removetext[pContext->rcode], orgfilename);

				//got the bitmap, don't copy to new Body-Field!
				dontcopy = TRUE;

				//getting memory for richtext field
				wStringLen = (WORD) strlen(szString) ;
				wStringBufLen = wStringLen + (wStringLen%2);//Space to allocate (must be an even number of bytes)

				dwCount = (WORD) (wStringBufLen + ODSLength(_CDTEXT));
				if (status = OSMemAlloc( 0, dwCount, &hcdtext ))
				{
					sprintf(szBuffer, "Error getting memory for body text");
					LogMessageTextSeverity(szBuffer, NOERROR,1);
					if (filename != NULL)
						free(filename);
					if (orgfilename != NULL)
						free(orgfilename);
					if (szString != NULL)
						free(szString);
					return(NOERROR);
				}


				//Write some text here in cdtext-structure
				cdtext.Header.Signature = SIG_CD_TEXT;
				cdtext.Header.Length = (WORD) dwCount ;

				fontData.fontFields.Face = FONT_FACE_SWISS;
				fontData.fontFields.Attrib = 0;
				fontData.fontFields.Color = NOTES_COLOR_RED;
				fontData.fontFields.PointSize = 12;

			    cdtext.FontID = fontData.fontId;

		        pSource = OSLock( BYTE, hcdtext );
				pcdtext = pSource;
				ODSWriteMemory(&pcdtext,_CDTEXT,&cdtext,1);
				memcpy( pcdtext,szString,wStringBufLen);

				sprintf(szBuffer,"written text: %.*s", MAXPATH, szString);
				LogMessageTextSeverity(szBuffer, NOERROR,3);

				if (pContext->fUseBody2 ==  FALSE)  // we did not previously overflow
				{
					if ( (DWORD)((pDest+dwCount) - pContext->pNewBody) >
										(pContext->dwNewBodyLen) ) // overflow!
					{
						pContext->fUseBody2 = TRUE;
						pDest = pContext->pNewBody2CurrPos;
					}
				}
				else //we previously overflowed NewBody. Continue with NewBody2.
				{
					pDest = pContext->pNewBody2CurrPos;
				}

				//copying of content
				memcpy( pDest, pSource, dwCount );


				//now go back and update the "CurrPos" pointers in the Context
				if (pContext->fUseBody2 == FALSE)
				{
					sprintf(szBuffer,"Advancing pointer");
					LogMessageTextSeverity(szBuffer, NOERROR,3);
					pContext->pNewBodyCurrPos += dwCount;
					if (dwCount % 2)
					{
						sprintf(szBuffer,"Advancing pointer by one");
						LogMessageTextSeverity(szBuffer, NOERROR,3);
						*(pContext->pNewBodyCurrPos) = 0;
						pContext->pNewBodyCurrPos++;
					}
				}
				else
				{
					sprintf(szBuffer,"Advancing pointer2");
					LogMessageTextSeverity(szBuffer, NOERROR,3);
					pContext->pNewBody2CurrPos += dwCount;
					if (dwCount % 2)
					{
						sprintf(szBuffer,"Advancing pointer2 by one");
						LogMessageTextSeverity(szBuffer, NOERROR,3);
						*(pContext->pNewBody2CurrPos) = 0;
						pContext->pNewBody2CurrPos++;
					}
				}

				if (filename != NULL)
					free(filename);
				if (orgfilename != NULL)
					free(orgfilename);
				if (szString != NULL)
					free(szString);

				return (NOERROR);

			}//if ( strcmp(filename, pContext->szFileName) == 0)
		}//if (cdHotSpot.Type == HOTSPOTREC_TYPE_FILE )
	}//if (RecordType == SIG_CD_HOTSPOTBEGIN)


	dwCount = (WORD) RecordLength;
	pSource = (BYTE *) RecordPtr;

	//pSource, pDest, and dwCount are ready for the memcpy. But first,
	//check that a record of size dwCount bytes will fit in the first
	//body (starting at pNewBodyCurrPos) without overflowing.

	if (pContext->fUseBody2 ==  FALSE)  // we did not previously overflow
	{
		if ( (DWORD)((pDest+dwCount) - pContext->pNewBody) > (pContext->dwNewBodyLen) ) // overflow!
		{
			pContext->fUseBody2 = TRUE;
			pDest = pContext->pNewBody2CurrPos;
		}
	}
	else //we previously overflowed NewBody. Continue with NewBody2.
	{
		pDest = pContext->pNewBody2CurrPos;
	}

	//Hier wird der eigentliche Inhalt kopiert.
	memcpy( pDest, pSource, dwCount);


	//now go back and update the "CurrPos" pointers in the Context
	if (pContext->fUseBody2 == FALSE)
	{
		sprintf(szBuffer,"Advancing pointer");
		LogMessageTextSeverity(szBuffer, NOERROR,3);
		pContext->pNewBodyCurrPos += dwCount;
		if (dwCount % 2)
		{
			sprintf(szBuffer,"Advancing pointer by one");
			LogMessageTextSeverity(szBuffer, NOERROR,3);
			*(pContext->pNewBodyCurrPos) = 0;
			pContext->pNewBodyCurrPos++;
		}
	}
	else
	{
		sprintf(szBuffer,"Advancing pointer2");
		LogMessageTextSeverity(szBuffer, NOERROR,3);
		pContext->pNewBody2CurrPos += dwCount;
		if (dwCount % 2)
		{
			sprintf(szBuffer,"Advancing pointer2 by one");
			LogMessageTextSeverity(szBuffer, NOERROR,3);
			*(pContext->pNewBody2CurrPos) = 0;
			pContext->pNewBody2CurrPos++;
		}
	}
    return NOERROR;
}





STATUS processSubject(NOTEHANDLE hNote, BOOL before, char virscan[256], int rcode)
{
	char	szBuffer[MAXPATH+MAXPATH];
	char	*szSubject=NULL;
	char	*szSubjectNew=NULL;
	BOOL 	field_found;
	WORD	field_len;
	char    field_text[512];

	field_found = NSFItemIsPresent (hNote, "SUBJECT", 7);
	if (field_found)
	{
		field_len = NSFItemGetText (hNote,
					"SUBJECT",
					field_text,
					sizeof (field_text) -1 );
		szSubject=(char *) malloc(field_len*sizeof(char) + 1);
		if (szSubject==NULL)
		{
			sprintf(szBuffer, "Error getting memory for subject");
			LogMessageTextSeverity(szBuffer, NOERROR,1);
			return(NOERROR);
		}
		sprintf(szSubject, "%s", field_text);
	} //end of if (field_found)
	else
	{
		sprintf(szBuffer, "No Subject-field found. Skipping.");
		LogMessageTextSeverity(szBuffer, NOERROR,3);
		return(NOERROR);
	}

	sprintf(szBuffer, "Manipulating Subject by adding %.*s for rc%.2i before=%.2i", MAXPATH, virscan, rcode, before);
	LogMessageTextSeverity(szBuffer, NOERROR,3);

	if (strstr(szSubject, virscan)==NULL)
	{

		// +2 == \0 und Leerzeichen
		szSubjectNew = (char *) malloc((field_len+strlen(virscan)+2) *sizeof(char));
		if (szSubjectNew==NULL)
		{
			sprintf(szBuffer, "Error getting memory for new subject");
			LogMessageTextSeverity(szBuffer, NOERROR,1);
			if ( szSubject!=NULL)
				free(szSubject);
			return(NOERROR);
		}

		if (before)
			sprintf(szSubjectNew, "%s %s", virscan, szSubject);
		else
			sprintf(szSubjectNew, "%s %s", szSubject, virscan);

		NSFItemSetText(hNote, "SUBJECT", szSubjectNew, (WORD) strlen(szSubjectNew));
	}
	if ( szSubject!=NULL)
		free(szSubject);
	if ( szSubjectNew!=NULL)
		free(szSubjectNew);
	return (NOERROR);
}


STATUS processBody(NOTEHANDLE hNote, char bodytext[256], int rcode)
{
	STATUS	serror;
	HANDLE	hBodyItem;
	DWORD	bodysize;
	char	szBuffer[MAXPATH+MAXPATH];

	if ( bodytext != "")
	{
		sprintf(szBuffer, "Manipulating Body by adding %.*s for rc%.2i", MAXPATH, bodytext, rcode);
		LogMessageTextSeverity(szBuffer, NOERROR,3);
	}
	else
	{
		sprintf(szBuffer, "Not Manipulating. Text is empty for rc%.2i", rcode);
		LogMessageTextSeverity(szBuffer, NOERROR,3);
	}

	//Adding warning to Mailtext
	if ( bodytext != "")
	{
		serror=MailCreateBodyItem(&hBodyItem, &bodysize);
		if (serror != NOERROR)
		{
			return(serror);
		}

		serror=MailAppendBodyItemLine(hBodyItem, &bodysize, "\n", (WORD) strlen("\n"));
		if (serror != NOERROR)
		{
			OSMemFree(hBodyItem);
			return(serror);
		}

		serror=MailAppendBodyItemLine(hBodyItem, &bodysize, bodytext, (WORD) strlen(bodytext));
		if (serror != NOERROR)
		{
			OSMemFree(hBodyItem);
			return(serror);
		}

		serror=MailAppendBodyItemLine(hBodyItem, &bodysize, "\n", (WORD) strlen("\n"));
		if (serror != NOERROR)
		{
			OSMemFree(hBodyItem);
			return(serror);
		}

		serror=MailAddBodyItemExt(hNote, hBodyItem, bodysize, NULL);
		if (serror != NOERROR)
		{
			OSMemFree(hBodyItem);
			return(serror);
		}

		OSMemFree(hBodyItem);
	}
	return (NOERROR);
}



#if defined(LINUX)
}
#endif
