/*
    PROGRAM:    mailscan

    FILE:       mailscan.c

    Author:	Hans-Georg Franke (hansgeorgfranke@yahoo.de)

	Enhancments by: Mikhail Cholokov

    This file is part of OpenMailScan.

    OpenMailScan is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    OpenMailScan is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with MailScan (see license.txt); if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
    */


#if defined(LINUX)
extern "C" {
#endif

#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>


#if defined POSIXIPC
#include <sys/ipc.h>
#include <sys/sem.h>
#include <sys/types.h>
#include <sys/shm.h>
#include <unistd.h>
#endif

#if defined (NT)
#include <process.h>
#endif

/*
 *  Domino and Notes header files
 */
#include <global.h>
#include <osmem.h>
#include <osfile.h>
#include <osenv.h>
#include <nsfdata.h>
#include <nsfdb.h>
#include <acl.h>
#include <nsfnote.h>
#include <editods.h>
#include <stdnames.h>
#include <mailserv.h>
#include <mail.h>
#include <textlist.h>


#include "log.h"
#include "mailscan.h"

/*========================== GLOBAL VARIABLES =============================*/

EMHANDLER         gHandlerProc;
HEMREGISTRATION	  hHandler;
HEMREGISTRATION	  hHandler1;
BOOL              gHooksRegistered=FALSE;
WORD              gRecursionID;
char              gTextBuffer[1024];
char              DataDir[MAXPATH+1];
char              FileName[MAXPATH+1];
char              szMailBoxPath[MAXPATH+1];
char              szString[MAXSPRINTF + 1];
char              UserName[MAXUSERNAME + 1];
char			  virscanencrypted[512];
char			  virscanaddinerror[512];
int			  	  virscanconvert;
char szVersion[256]="MailScan ExtensionManager V0.99f";


#if defined(NT)
CRITICAL_SECTION    gCriticalSection;
#endif

/************************* End of Global Variables *************************
****************************************************************************/

#if defined(NT)
BOOL WINAPI DllMain( HINSTANCE hInstance, DWORD fdwReason, LPVOID lpReserved )
{
  STATUS error=NOERROR;

  switch(fdwReason)
  {
    case DLL_PROCESS_ATTACH:
      InitializeCriticalSection(&gCriticalSection);
      gHandlerProc = (EMHANDLER)MakeProcInstance((FARPROC)EMHandlerProcMailScan, hInstance);
      break;

    case DLL_PROCESS_DETACH:
		gHooksRegistered = FALSE;
		// Free procedure instance

		FreeProcInstance( gHandlerProc );

		error=DeregisterEntries();
		if (error){
			LogMessageTextSeverity("Deregisteration isn't successfull\n", NOERROR,3);
			break;
			}
	  //error = DeregisterEntry();
      DeleteCriticalSection(&gCriticalSection);
      break;
  }

  return( TRUE );

  UNREFERENCED_PARAMETER(lpReserved);
}

#endif






DLL_EXPORT_PREFIX STATUS LNPUBLIC DLL_EXPORT_INFIX MainEntryPoint (void)
{
    STATUS    error = NOERROR;
	char szBuffer[1024]="";

	char sLoadedTask[MAXENVVALUE]="";
	char sLoadWithTask[MAXENVVALUE]="";
	BOOL ifFound = FALSE;


	//Cholokov:
	// search for MailScan_LoadWith=
	// I've checked "SMTP Server". If you'd like all server processes registration
	// don't add MailScan_LoadWith parameter into notes.ini
	// Franke:
	// If your server serves user, you also need "Server"
	//

	DebugLevel=3;

	OSLoadString(NULLHANDLE, PKG_ADDIN+0,sLoadedTask,MAXENVVALUE);
	sprintf(szBuffer, "Loaded task *%s*", sLoadedTask);
	LogMessageTextSeverity(szBuffer, NOERROR,3);

	ifFound = OSGetEnvironmentString("MailScan_LoadWith",sLoadWithTask,MAXENVVALUE);

	if (ifFound){
		int n=0;
		char seps[]   = ",";
		char *token;
		BOOL righttask=FALSE;
		BOOL emptytasknames=FALSE;
		//Check, one or more entries in commandline
		
		token = strtok( sLoadWithTask, seps );
		strlwr(sLoadedTask);
		strlwr(token);

		while( token != NULL )
		{

			sprintf(szBuffer, "will load only with *%s* task", token);
			LogMessageTextSeverity(szBuffer, NOERROR,3);


			if ( (strcmp(sLoadedTask,token) == 0)){
				sprintf(szBuffer, "Right task, will load: %s - %s",sLoadedTask,token);
				LogMessageTextSeverity(szBuffer, NOERROR,3);
				righttask = TRUE;
			}
			if ( (strcmp("empty",token) == 0)){
				sprintf(szBuffer, "Right task, will load: empty - %s",token);
				LogMessageTextSeverity(szBuffer, NOERROR,3);
				emptytasknames = TRUE;
			}
			
			token = strtok( NULL, seps );
			if (token != NULL)
				strlwr(token);
		}

		if ( ((int) strlen(sLoadedTask) == 0) && emptytasknames )  {
			sprintf(szBuffer, "Size of Taskname is 0 - Loading for this task");
			LogMessageTextSeverity(szBuffer, NOERROR,3);
			righttask = TRUE;
		}
		if ( !righttask ){
			sprintf(szBuffer, "Loaded task \"%s\" is different - abbort loading", sLoadedTask);
			LogMessageTextSeverity(szBuffer, NOERROR,3);
			return( NOERROR );
		}
	}
	else
	{
		sprintf(szBuffer, "will loaded with any task");
		LogMessageTextSeverity(szBuffer, NOERROR,3);
	}

	sprintf(szBuffer, "%s", szVersion);
	LogMessageTextConSeverity(szBuffer, NOERROR,0);

	//From here use desired debuglevel!
	DebugLevel=OSGetEnvironmentInt("MailScan_ExtDebugLevel");
	sprintf(szBuffer, "DebugLevel: %i", DebugLevel);
	LogMessageTextConSeverity(szBuffer, NOERROR,3);

	#if defined(LINUX)
		sprintf(szBuffer, "PID>%d", getpid());
	#else
		sprintf(szBuffer, "PID>%d", _getpid() );
	#endif
	LogMessageTextSeverity(szBuffer, NOERROR,3);


	if (OSGetEnvironmentString("MailScan_Encrypted", virscanencrypted, sizeof(virscanencrypted)-1))
	{
		sprintf(szBuffer, "Tag encrypted Mail with %s", virscanencrypted);
		LogMessageTextSeverity(szBuffer, NOERROR,3);
	}
	else
	{
		sprintf(szBuffer, "Tag for encrypted Mail not found");
		LogMessageTextSeverity(szBuffer, NOERROR,3);
		sprintf(szBuffer, "Using default Tag: \"Mail encrypted -> not scanned\"");
		sprintf(virscanencrypted, "\"Mail encrypted -> not scanned\"");
		LogMessageTextSeverity(szBuffer, NOERROR,3);
	}


	if (OSGetEnvironmentString("MailScan_AddinError", virscanaddinerror, sizeof(virscanaddinerror)-1))
	{
		sprintf(szBuffer, "AddinError: Tag Mail with %s", virscanaddinerror);
		LogMessageTextSeverity(szBuffer, NOERROR,3);
	}
	else
	{
		sprintf(szBuffer, "Tag for AddinError not found");
		LogMessageTextSeverity(szBuffer, NOERROR,3);
		sprintf(szBuffer, "Using default Tag: \"Mail not scanned -> AddinError\"");
		sprintf(virscanaddinerror, "\"Mail not scanned -> AddinError\"");
		LogMessageTextSeverity(szBuffer, NOERROR,3);
	}

	LogMessageTextSeverity("Start registering  ....", NOERROR,3);

	#if defined (UNIX)
	strcat(FileName, "/");
	#else
	strcat(FileName, "\\");
	#endif

	//
	//    When run on a server the dll is called multiple times. the flag
	//    keeps the main code from being executed more than once.
	//

    if ( gHooksRegistered )
	 return(NOERROR);
	gHooksRegistered = TRUE;

    // Get proc instance for the handler callback.
	#if defined (NT)
	// Done in the DllMain function at startup.
	#else
	gHandlerProc = EMHandlerProcMailScan;
	#endif

	//  Next get a recursion ID  for the run

    error = EMCreateRecursionID( &gRecursionID );

	if (error == NOERROR)
		error=RegisterEntries();

    if (error != NOERROR){
		LogMessageTextSeverity("Register Unsuccessful", NOERROR,3);
		return(error);
	}

    LogMessageTextSeverity("Registered",NOERROR,3);
    return( NOERROR );
}

/*=========================================================================*/





STATUS LNPUBLIC FAR EMHandlerProcMailScan( EMRECORD FAR * pExRecord )
{

WORD		field_len;
char		field_text[500];
BOOL		field_found=FALSE;
char		szBuffer[700];

#if defined POSIXIPC
key_t bakey= _HGF_SEMAPHORE_LINUX;
int bansems = 1;
int basemflg = 0600;
int basemid;
#else
HANDLE hMutex;
#endif

  switch (pExRecord->EId)
  {
    case EM_TERMINATENSF:
    	{
			// remove blocking from the system
			// Normaly not needed. Just for removing semaphore if addin-task crashes
			LogMessageTextSeverity("Starting Event processing for EM_TerminateNSF...", NOERROR,3);
			removesem();
    		break;
	    }

	case EM_NSFNOTEUPDATEMAILBOX:
    	{
		HANDLE hNote;
		void   *pViewDesc;
		WORD   Flags;
		BOOL   *pModified;
		VARARG_PTR ap;

		// get the arguments
		ap = pExRecord->Ap;
		hNote = VARARG_GET (ap, HANDLE);
		pViewDesc = VARARG_GET (ap, VOID *);
		Flags = VARARG_GET (ap, int);
		pModified = VARARG_GET (ap, BOOL *);

		LogMessageTextSeverity("Starting Event processing for EM_NSFNoteUpdateMailbox...", NOERROR,3);

		#if defined POSIXIPC

		//Gettting mutex
		if ( ( basemid = semget(bakey, 1, 0600 | IPC_CREAT | IPC_EXCL) ) == -1  )
		{
			LogMessageTextSeverity("1- Addin is running", NOERROR,0);
		}
		else
		{

			LogMessageTextSeverity("Got mutex, therefore: Addin isn't running", NOERROR,0);
			semctl(basemid, 0, IPC_RMID);
			field_found = NSFItemIsPresent (hNote, "SUBJECT", (WORD) strlen ("SUBJECT"));
			if (field_found)
				{
				 field_len = NSFItemGetText (hNote, "SUBJECT", field_text, sizeof (field_text));
    			 sprintf(szBuffer, "%s %s", field_text, virscanaddinerror);
				 NSFItemSetText(hNote, "SUBJECT", szBuffer, (WORD) strlen(szBuffer));
				 sprintf(szBuffer, "Memo Subjects: %s ", field_text);
				 LogMessageTextSeverity(szBuffer, NOERROR,3);
				 }
			break;
		}

		LogMessageTextSeverity("Starting Event processing...", NOERROR,3);
		#else

		hMutex = OpenMutex( MUTEX_ALL_ACCESS, FALSE,"HGFMailScanMutex");
		if (hMutex == NULL)
		{
			//Addin isn't running
			LogMessageTextSeverity("Addin ins't running", NOERROR,0);
			field_found = NSFItemIsPresent (hNote, "SUBJECT", (WORD) strlen ("SUBJECT"));
			if (field_found)
				{
					field_len = NSFItemGetText (hNote, "SUBJECT", field_text, sizeof (field_text));
					sprintf(szBuffer, "%s %s", field_text, virscanaddinerror);
					NSFItemSetText(hNote, "SUBJECT", szBuffer, (WORD) strlen(szBuffer));
					sprintf(szBuffer, "Inhalt des Subjects: %s ", field_text);
					LogMessageTextSeverity(szBuffer, NOERROR,3);
					}
			break;
		}
		else
		{
			LogMessageTextSeverity("Addin is running", NOERROR,0);
		}

		#endif


		LogMessageTextSeverity("Starting Eventhandling", NOERROR,2);

		// check error code
		if (pExRecord->Status != NOERROR)
		{
			break;
		}

		if (pExRecord->NotificationType == EM_BEFORE)
		{

			#if defined(LINUX)
				sprintf(szBuffer, "PID>%d", getpid());
			#else
				sprintf(szBuffer, "PID>%d", _getpid());
			#endif
			LogMessageTextSeverity(szBuffer, NOERROR,3);

			LogMessageTextSeverity("Handling Event", NOERROR,3);


			field_len = NSFItemGetText ( hNote,"Recipients", field_text, sizeof (field_text));
			if ( field_len == 0)
			{
				//recipients-item not found => router handled message - ignoring
				LogMessageTextSeverity("No Recipients-Field found", NOERROR,3);
				break;
			}


			//If Document contains Field "MailScanNoteProcessed", then reading content
			field_len = NSFItemGetText ( hNote,"MailScanNoteProcessed", field_text, sizeof (field_text));

			if ( field_len == 0)
			{
				//Nothing to do here
				//Must be scanned!!
			}
			else
			{
				LogMessageTextSeverity("MailScanNoteProcessed found", NOERROR,3);
				if ( strncmp(field_text,"TRUE",field_len)==0 )
				{
					//Doc is already processed
					LogMessageTextSeverity("MailScanNoteProcessed==TRUE found", NOERROR,3);
					break;
				}
				if ( strncmp(field_text,"RUN", field_len)==0 )
				{
					//DocProcessing runs
					LogMessageTextSeverity("MailScanNoteProcessed==RUN found", NOERROR,3);
					break;
				}
			}

			LogMessageTextSeverity("Note preprocess begining...", NOERROR,3);

			// Testing for attachments in memo
			if (testAttachment(hNote) == NOERROR)
			{
				BOOL 	issigned;
				BOOL 	isencrypted;
				STATUS	sstatus;

				LogMessageTextSeverity("Attachment found", NOERROR,0);

				sstatus = NSFNoteIsSignedOrSealed(hNote,
						&issigned, &isencrypted);

				if (isencrypted == TRUE)
				{
				  LogMessageTextSeverity("Mail is encrypted", NOERROR,0);
				  field_found = NSFItemIsPresent (hNote, "SUBJECT", (WORD) strlen ("SUBJECT"));
				  if (field_found)
				  {
					field_len = NSFItemGetText (hNote, "SUBJECT", field_text,
								sizeof (field_text));

					sprintf(szBuffer, "%s %s", field_text, virscanencrypted);
					NSFItemSetText(hNote, "SUBJECT", szBuffer, (WORD) strlen(szBuffer));
					sprintf(szBuffer, "Inhalt des Subjects: %s ", field_text);
					LogMessageTextSeverity(szBuffer, NOERROR,3);
				  }
				  else
				  {
					  LogMessageTextSeverity("No Subject found", NOERROR,3);
				  }
				}
				else
				{
				  LogMessageTextSeverity("Handling mail", NOERROR,0);
				  NSFItemSetText(hNote, "MailScanNoteProcessed", "Pending", 7);
				  NSFItemSetText(hNote, "RoutingState", "HOLD", 4);
				  NSFItemSetText(hNote, "FailureReason", "Processing Note by mailscan", (WORD) strlen("Processing Note by mailscan"));
				}
			}
			else
			{
				LogMessageTextSeverity("No attachment found - ready", NOERROR,0);
				NSFItemSetText(hNote, "MailScanNoteProcessed", "TRUE", 4);
			}

		} // end of EM_BEFORE
		break;
	} // end of case

  } // end of switch

  LogMessageTextSeverity("Returning normally", NOERROR,3);
  return( ERR_EM_CONTINUE );
}



STATUS testAttachment( HANDLE hNote){
	STATUS sstatus=NOERROR;
	DBHANDLE hdb;
	HANDLE hNoteNew;
	DWORD  Noteid;
	BLOCKID bid;
	WORD wType;
	BLOCKID bidValue;
	DWORD dwLength;
	int virscanconvert;
	char szBuffer[256];
	WORD retAccessLevel;
	WORD retAccessFlag;

			//Testing for inline-attached mails if configured

			virscanconvert = OSGetEnvironmentInt("MailScan_ConvertToCDOn");
			sprintf(szBuffer,"ConvertToCDOn: %u\n", virscanconvert);
			LogMessageTextSeverity(szBuffer, NOERROR,3);

			NSFNoteGetInfo(hNote, _NOTE_DB, &hdb);
			sprintf(szBuffer,"dbhandle: %hu\n", hdb);
			LogMessageTextSeverity(szBuffer, NOERROR,3);

			NSFDbAccessGet(hdb, &retAccessLevel, &retAccessFlag);
			sprintf(szBuffer,"AccessLevel: %u\n", retAccessLevel);
			LogMessageTextSeverity(szBuffer, NOERROR,3);

			if ( (virscanconvert == 1) && (retAccessLevel > ACL_LEVEL_AUTHOR) )
			{
				//Not found or String or set to FALSE == 0
				LogMessageTextSeverity("Try to reopen Document\n", NOERROR,3);

				sstatus = NSFNoteUpdate(hNote, UPDATE_FORCE);
				if (sstatus != NOERROR)
				{
					LogMessageTextSeverity("Error saving Document\n", NOERROR,0);
					return(sstatus);
				}
				LogMessageTextSeverity("Document saved\n", NOERROR,3);

				NSFNoteGetInfo(hNote, _NOTE_ID, &Noteid);
				sprintf(szBuffer,"NOTEID: %u\n", Noteid);
				LogMessageTextSeverity(szBuffer, NOERROR,3);

				sstatus = NSFNoteOpen(hdb, Noteid,0, &hNoteNew);
				if (sstatus != NOERROR)
				{
					LogMessageTextSeverity("Error reopening Document\n", NOERROR,0);
					return(sstatus);
				}


				sstatus = NSFItemInfo(hNoteNew,
							ITEM_NAME_ATTACHMENT,
							(WORD) strlen(ITEM_NAME_ATTACHMENT),
							&bid, &wType, &bidValue, &dwLength);

				NSFNoteClose(hNoteNew);

			}
			else
			{
				LogMessageTextSeverity("Remark: Not scanning possible inline attachments\n", NOERROR,3);

				sstatus = NSFItemInfo(hNote,
							ITEM_NAME_ATTACHMENT,
							(WORD) strlen(ITEM_NAME_ATTACHMENT),
							&bid, &wType, &bidValue, &dwLength);
			}

			return(sstatus);
}



void removesem()
{
	#if defined(POSIXIPC)
	//int basemid;
	//if( (basemid=semget(_HGF_EXT_SEMAPHORE,0,0666)) != -1){
	//	semctl(basemid, 0, IPC_RMID);
	//}

	#else

	#endif
}



//
// RegisterEntry: Attempts to Register one entry.
//
//   out: 0 if ok, returned error otherwise
//
STATUS RegisterEntries()
{

    STATUS error = NOERROR;

    LogMessageTextSeverity("Registering hHandler for NSFNoteUpdateMailbox...", NOERROR,3);

    error = EMRegister(EM_NSFNOTEUPDATEMAILBOX,
                           EM_REG_BEFORE,
	                   (EMHANDLER)gHandlerProc,
	                   gRecursionID,
	                   &hHandler);

    if (error != NOERROR)
    {
		LogMessageTextSeverity("Error registering hHandler...", NOERROR,3);
    	return(error);
	}


	LogMessageTextSeverity("Registering hHandler1 fo TerminateNSF...", NOERROR,3);
    error = EMRegister(EM_TERMINATENSF,
                           EM_REG_BEFORE,
	                   (EMHANDLER)gHandlerProc,
	                   gRecursionID,
	                   &hHandler1);

    if (error != NOERROR)
    {
		LogMessageTextSeverity("Error registering hHandler1...", NOERROR,3);
    	return(error);
	}


    LogMessageTextSeverity("Registering hHandler and hHandler1 completed...", NOERROR,3);
    return(error);

}




//
// DeregisterEntry: Attempts to Deregister one entry.
//
//   out: 0 if ok, returned error otherwise
//
STATUS DeregisterEntries()
{
    STATUS error = NOERROR;

    LogMessageTextSeverity("Deregistering hHandler...", NOERROR,3);

    error = EMDeregister(hHandler);
    if (error != NOERROR)
    {
		LogMessageTextSeverity("Error deregistering hHandler", NOERROR,3);
    	return(error);
	}

    LogMessageTextSeverity("Deregistering hHandler1...", NOERROR,3);

    error = EMDeregister(hHandler1);
    if (error != NOERROR)
    {
		LogMessageTextSeverity("Error deregistering hHandler1", NOERROR,3);
    	return(error);
	}

    LogMessageTextSeverity("Deregistering complete...", NOERROR,3);

    return(error);

}



DLL_EXPORT_PREFIX STATUS LNPUBLIC DLL_EXPORT_INFIX ExtClear (void)
{

	  return (DeregisterEntries());
}

#if defined(LINUX)
}
#endif
